﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace HL_1
{
    public partial class MainForm : Form
    {
        public System.Windows.Forms.MdiClient mc;
        public ListForm ListForm;
        public DispForm1 DispForm1;
        public DispForm2 DispForm2;
        public DispForm3 DispForm3;
        public DispForm4 DispForm4;
        public AboutBox1 AboutBox1;


        public cCell Cell;
        public double dt = 0.00001;
        public double TimeElp;			//elapsed time in a frame
        public double TimeElp_Sht;		//elapsed time for short display
        public double TimeElp_Lng;		//elapsed time for long display
        public double ElapsedTime_cyc;

        public bool BlnWriteSimData;
        public bool BlnStopFlag;
        public bool BlnPause;
        public bool BlnParListForm;

        private string InitFileName;
        private string SimFileName;
        private string SimFileName_beat;
        private string LCFileName;
        private string LCFNameForProtocol = "testLCFnameP";

        private int PeakFlag = 0;
        public double dVdTVal;
        public double VmOld;
        public Pd.TdVdTPeak[] dVdTPeak;
        public double HR;
        public double CL;

        private bool BlnProtocolOn = false;

        public MainForm()
        {
            InitializeComponent();

            Cell = new cCell();

            ListForm = new ListForm(this);
            ListForm.MdiParent = this;
            ListForm.Text = "Parameter List";

            DispForm1 = new DispForm1(this);
            DispForm1.MdiParent = this;
            DispForm2 = new DispForm2(this);
            DispForm2.MdiParent = this;
            DispForm3 = new DispForm3(this);
            DispForm3.MdiParent = this;
            DispForm4 = new DispForm4(this);
            DispForm4.MdiParent = this;
            AboutBox1 = new AboutBox1(this);
            

            dVdTPeak = new Pd.TdVdTPeak[2];
            for (int i = 0; i <= 1; i++)
            {
                dVdTPeak[i] = new Pd.TdVdTPeak();
            };
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            DispForm1.Show();
            DispForm2.Show();
            DispForm3.Show();
            DispForm4.Show();
            ListForm.Show();

            DispForm1.Top = 0;
            DispForm2.Top = 0;
            DispForm3.Top = 0;
            DispForm4.Top = 0;
            ListForm.Top = 0;

            DispForm1.Left = 0;
            DispForm2.Left = DispForm1.Width;
            DispForm3.Left = DispForm1.Width + DispForm2.Width;
            DispForm4.Left = DispForm1.Width + DispForm2.Width + DispForm3.Width;
            ListForm.Left = DispForm1.Width + DispForm2.Width + DispForm3.Width + DispForm4.Width;

            ResetToolStripMenuItem.Enabled = false;
            StartToolStripMenuItem.Enabled = false;
            StopToolStripMenuItem.Enabled = false;
            SaveToolStripMenuItem.Enabled = false;

            GetNewSettings();

            DispForm1.SendImageBmpToPbox();
            DispForm2.SendImageBmpToPbox();
            DispForm3.SendImageBmpToPbox();
            DispForm4.SendImageBmpToPbox();
        }

        private void SetSimDataFile(ref string strFileName)
        {
            //'SaveFileDialogクラスのインスタンスを作成
            SaveFileDialog sfd = new SaveFileDialog();

            string FName = "test";

            // ファイルのフィルタを設定する
            sfd.Filter = "テキスト ファイル|*.csv;*.log|すべてのファイル|*.*";

            // ファイルの種類 の初期設定を 1 番目に設定する (初期値 1)
            sfd.FilterIndex = 1;

            // ダイアログのタイトルを設定する
            sfd.Title = "保存先のファイルを選択してください";

            // ダイアログボックスを閉じる前に現在のディレクトリを復元する (初期値 false)
            sfd.RestoreDirectory = true;

            // 存在しないファイルを指定した場合は、新しく作成するかどうかの問い合わせを表示する (初期値 false)
            sfd.CreatePrompt = true;

            // 存在しているファイルを指定した場合は、上書きするかどうかの問い合わせを表示する (初期値 true)
            sfd.OverwritePrompt = true;

            //'存在しないパスが指定されたとき警告を表示する。デフォルトでTrueなので指定する必要はない
            sfd.CheckPathExists = true;

            // ダイアログを表示し、戻り値が [OK] の場合は、選択したファイルを表示する
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                FName = sfd.FileName;
                if (System.IO.File.Exists(FName))
                {
                    System.IO.File.Delete(FName);
                };
            }
            else
            {
                FName = "test";
            }
            strFileName = FName;
        }

        private void GetNewSettings()
        {
            Cell.MSet.StimCycle = Double.Parse(Txt_OneCycle.Text);
            Cell.MSet.SDispTimeMax = Cell.MSet.StimCycle;       // Double.Parse(Txt_ShtDispTimeMax.Text);


            if (ComboBox2.SelectedIndex == 0)
            {
                Cell.MSet.LDispTimeMax = Double.Parse(Txt_LngDispTimeMax.Text); // msec	
            }
            else if (ComboBox2.SelectedIndex == 1)
            {
                Cell.MSet.LDispTimeMax = Double.Parse(Txt_LngDispTimeMax.Text) * 1000.0;  // sec . msec
            }
            else if (ComboBox2.SelectedIndex == 2)
            {
                Cell.MSet.LDispTimeMax = Double.Parse(Txt_LngDispTimeMax.Text) * 1000.0 * 60.0;  // min . msec	
            }
            else if (ComboBox2.SelectedIndex == 3)
            {
                Cell.MSet.LDispTimeMax = Double.Parse(Txt_LngDispTimeMax.Text) * 1000.0 * 60.0 * 60.0; // hr . msec	
            }

           

            if (ComboBox1.SelectedIndex == 0)
            {
                Cell.MSet.SimTimeMax = Double.Parse(Txt_SimeTimeMax.Text);
            }
            else if (ComboBox1.SelectedIndex == 1)
            {
                Cell.MSet.SimTimeMax = Double.Parse(Txt_SimeTimeMax.Text) * 1000.0;  //' sec	
            }
            else if (ComboBox1.SelectedIndex == 2)
            {
                Cell.MSet.SimTimeMax = Double.Parse(Txt_SimeTimeMax.Text) * 1000.0 * 60.0; // min	
            }
            else if (ComboBox1.SelectedIndex == 3)
            {
                Cell.MSet.SimTimeMax = Double.Parse(Txt_SimeTimeMax.Text) * 1000.0 * 60.0 * 60.0; // hr
            }

            DispForm1.DispXmax(Cell.MSet.LDispTimeMax);
            DispForm2.DispXmax(Cell.MSet.LDispTimeMax);
            DispForm3.DispXmax(Cell.MSet.LDispTimeMax);
            DispForm4.DispXmax(Cell.MSet.LDispTimeMax);

            DispForm1.GetNewSettings();
            DispForm2.GetNewSettings();
            DispForm3.GetNewSettings();
            DispForm4.GetNewSettings();
        }

        private void ModiValueCells(cCell myCell)
        {
            myCell.ModiValues(ref ListForm, myCell);
        }

        void SimCells()
        {
            double TimeElp_Plot = 0.0;
            double PlotInt_Sht = 0.0;
            double TimeElp_DispSht = 0.0;
            double TimeElp_SimSave = 0.0;
            double SimSaveIntV = 0.0;
            double RereshIntV = 0.0;
            double TimeRereshIntV = 0.0;

            int SampCount = 0;
            int SampCount_beat = 0;
            int NumOfLDisp = 1;

            string Hstr;
            string Mstr;
            string Sstr;
            double Hdbl;
            double Mdbl;
            int Sint;
            double TimeSec;
            double TimeSimStart;
            bool BlnLastCycle;

            //計算時間計測用
            System.Diagnostics.Stopwatch sw = new System.Diagnostics.Stopwatch();

            SimFileName = "SimData";
            LCFileName = "LastCycleData";
            BlnPause = false;

            GetNewSettings();

            DispForm1.EraseGraphs();
            DispForm2.EraseGraphs();
            DispForm3.EraseGraphs();
            DispForm4.EraseGraphs();

            ModiValueCells(Cell);
            SetInitMinMax(ref Cell);

            BlnStopFlag = false;
            BlnPause = false;
            PlotInt_Sht = double.Parse(Txt_PlotIntv.Text);
            RereshIntV = double.Parse(Txt_RefreshIntv.Text);
            SimSaveIntV = double.Parse(Txt_SimSaveIntV.Text);
            Pd.BlnFirstPlot_SDisp = true;
            Pd.BlnFirstPlot_LDisp = true;
            NumOfLDisp = 1;
            TimeSimStart = TimeElp;
            BlnLastCycle = false;

            if (Rbn_SaveSimData.Checked == true)
            {
                SetSimDataFile(ref SimFileName);
                SimFileName_beat = System.IO.Path.GetDirectoryName(SimFileName) + '\\'
                                 + System.IO.Path.GetFileNameWithoutExtension(SimFileName) + "_beat.csv";
            }
            else if (Rbn_SaveLC.Checked && BlnProtocolOn == false)
            {
                SetSimDataFile(ref LCFileName);
            }

            while (TimeElp <= Cell.MSet.SimTimeMax + TimeSimStart)
            {
                TimeElp_Sht = 0;
                Pd.BlnFirstPlot_SDisp = true;

                if (TimeElp_Lng > Cell.MSet.LDispTimeMax)
                {
                    if (Cbx_OverWriteDisps.Checked == false)
                    {
                        DispForm1.EraseGraphs();
                        DispForm2.EraseGraphs();
                        DispForm3.EraseGraphs();
                        DispForm4.EraseGraphs();
                    }

                    TimeElp_Lng = 0;
                    Pd.BlnFirstPlot_LDisp = true;
                    NumOfLDisp = NumOfLDisp + 1;
                };

                if (TimeElp >= Cell.MSet.SimTimeMax + TimeSimStart - Cell.MSet.StimCycle) BlnLastCycle = true;

                sw.Start();

                while (TimeElp_Sht < Cell.MSet.SDispTimeMax)
                {
                    Cell.Integrate(ref dt, ref Cell.TVc, Cell);

                    if (TimeElp_Plot > PlotInt_Sht)
                    {
                        if (Cbx_ShowDispForms.Checked == true)
                        {
                            DispForm1.PlotSimDisp(TimeElp_Lng, Cell);
                            DispForm2.PlotSimDisp(TimeElp_Lng, Cell);
                            DispForm3.PlotSimDisp(TimeElp_Lng, Cell);
                            DispForm4.PlotSimDisp(TimeElp_Lng, Cell);
                        }

                        if (Pd.BlnFirstPlot_SDisp == true)
                        {
                            Pd.BlnFirstPlot_SDisp = false;
                        };
                        if (Pd.BlnFirstPlot_LDisp == true)
                        {
                            Pd.BlnFirstPlot_LDisp = false;
                        };
                        TimeElp_Plot = 0;
                    };

                    if (TimeRereshIntV > RereshIntV)
                    {
                        if (Cbx_ShowDispForms.Checked == true)
                        {
                            DispForm1.SendImageBmpToPbox();
                            DispForm2.SendImageBmpToPbox();
                            DispForm3.SendImageBmpToPbox();
                            DispForm4.SendImageBmpToPbox();
                        }

                        TimeRereshIntV = 0.0;
                    }

                    if (Rbn_SaveSimData.Checked)
                    {
                        if ((TimeElp_SimSave >= SimSaveIntV))
                        {
                            SaveSimData(SimFileName, TimeElp, SampCount);
                            ++SampCount;
                            TimeElp_SimSave = 0.0;
                        };
                    }
                    else if (Chk_SaveLastCycle.Checked && BlnLastCycle == true)
                    {
                        if ((TimeElp_SimSave >= SimSaveIntV))
                        {
                            if (BlnProtocolOn == false)
                            {
                                SaveSimData(LCFileName, TimeElp, SampCount);
                            }
                            else
                            {
                                SaveSimData(LCFNameForProtocol, TimeElp, SampCount);
                            };

                            ++SampCount;
                            TimeElp_SimSave = 0.0;
                        };
                    };

                    if (ElapsedTime_cyc >= Cell.MSet.StimCycle)
                    {
                        if (Rbn_SaveSimData.Checked)
                        {
                            SaveSimDataByBeat(SimFileName_beat, TimeElp, SampCount_beat);
                            ++SampCount_beat;
                        };

                        DispMinMax(Cell);
                        if (Cbx_ShowDispForms.Checked == true)
                        {
                            DispForm1.DispLabels(Cell);
                            DispForm2.DispLabels(Cell);
                            DispForm3.DispLabels(Cell);
                            DispForm4.DispLabels(Cell);
                        }

                        SetInitMinMax(ref Cell);

                        ElapsedTime_cyc = 0;
                    }
                    else
                    {
                        FindMinMax(ref Cell);
                    };

                    //increment elapsed time
                    TimeElp = TimeElp + dt;
                    TimeElp_Sht = TimeElp_Sht + dt;
                    TimeElp_Lng = TimeElp_Lng + dt;
                    ElapsedTime_cyc = ElapsedTime_cyc + dt;
                    TimeElp_Plot = TimeElp_Plot + dt;
                    TimeElp_DispSht = TimeElp_DispSht + dt;
                    TimeElp_SimSave = TimeElp_SimSave + dt;
                    TimeRereshIntV = TimeRereshIntV + dt;


                } //End of 2nd while

                sw.Stop();
                lbl_SimRealRatio.Text = String.Format("{0,6:#.#0}", TimeElp_Sht / sw.ElapsedMilliseconds);
                sw.Reset();

                Cell.DispValues(ref ListForm);

                TimeSec = TimeElp / 1000;
                lbl_Time.Text = TimeSec.ToString("F1");   //Format(TimeSec, "#.#")
                Hdbl = Math.Truncate(TimeSec / 60.0 / 60.0);
                Mdbl = Math.Truncate((TimeSec - Hdbl * 60 * 60) / 60);
                Sint = ((int)(TimeSec - Hdbl * 60 * 60)) % 60;

                Hstr = String.Format("{0,3:00}", Hdbl);
                Mstr = String.Format("{0,3:00}", Mdbl);
                Sstr = String.Format("{0,3:00}", Sint);

                lblTime_min.Text = Hstr + ":" + Mstr + ":" + Sstr;
                lbl_Time.Refresh();
                lblTime_min.Refresh();

                Application.DoEvents();
                if (BlnStopFlag == true)
                {
                    sw.Stop();
                    ElapsedTime_cyc = 0;
                    break;
                };
            }   //End of 1st while

            TimeSimStart = TimeElp;
            TimeElp_Sht = 0.0;
            ElapsedTime_cyc = 0.0;

            if (Rbn_SaveSimData.Checked)
            {
                if (SampCount > 0)
                {
                    WriteSimData(SimFileName, TimeElp, SampCount);
                    WriteSimDataBybeat(SimFileName_beat, TimeElp, SampCount_beat);
                };
            }
            else if (Rbn_SaveLC.Checked && BlnLastCycle == true)
            {
                if (SampCount > 0)
                {
                    if (BlnProtocolOn == false)
                    {
                        WriteSimData(LCFileName, TimeElp, SampCount);
                    }
                    else
                    {
                        WriteSimData(LCFNameForProtocol, TimeElp, SampCount);
                    }
                };
            };
        }

        private void SetInitMinMax(ref cCell myCell)
        {
            myCell.Vm_Min = myCell.TVc[Pd.IdxVm];
            myCell.Vm_Max = myCell.TVc[Pd.IdxVm];

            myCell.Cyt.Cai_Min = myCell.TVc[Pd.IdxCai];
            myCell.Cyt.Cai_Max = myCell.TVc[Pd.IdxCai];

            myCell.JS.Cajs_Min = myCell.TVc[Pd.IdxCajs];
            myCell.JS.Cajs_Max = myCell.TVc[Pd.IdxCajs];

            myCell.SL.Casl_Min = myCell.TVc[Pd.IdxCasl];
            myCell.SL.Casl_Max = myCell.TVc[Pd.IdxCasl];

            myCell.SR.Casr_Min = myCell.TVc[Pd.IdxCasr];
            myCell.SR.Casr_Max = myCell.TVc[Pd.IdxCasr];

            myCell.Mitochondria.Camit_Min = myCell.TVc[Pd.InxCamit];
            myCell.Mitochondria.Camit_Max = myCell.TVc[Pd.InxCamit];
        }

        private void FindMinMax(ref cCell myCell)
        {
            if (myCell.Vm_Min >= myCell.TVc[Pd.IdxVm])
            {
                myCell.Vm_Min = myCell.TVc[Pd.IdxVm];
            };
            if (myCell.Vm_Max <= myCell.TVc[Pd.IdxVm])
            {
                myCell.Vm_Max = myCell.TVc[Pd.IdxVm];
            };

            if (myCell.Cyt.Cai_Min >= myCell.TVc[Pd.IdxCai])
            {
                myCell.Cyt.Cai_Min = myCell.TVc[Pd.IdxCai];
            };
            if (myCell.Cyt.Cai_Max <= myCell.TVc[Pd.IdxCai])
            {
                myCell.Cyt.Cai_Max = myCell.TVc[Pd.IdxCai];
            };

            if (myCell.JS.Cajs_Min >= myCell.TVc[Pd.IdxCajs])
            {
                myCell.JS.Cajs_Min = myCell.TVc[Pd.IdxCajs];
            };
            if (myCell.JS.Cajs_Max <= myCell.TVc[Pd.IdxCajs])
            {
                myCell.JS.Cajs_Max = myCell.TVc[Pd.IdxCajs];
            };

            if (myCell.SL.Casl_Min >= myCell.TVc[Pd.IdxCasl])
            {
                myCell.SL.Casl_Min = myCell.TVc[Pd.IdxCasl];
            };
            if (myCell.SL.Casl_Max <= myCell.TVc[Pd.IdxCasl])
            {
                myCell.SL.Casl_Max = myCell.TVc[Pd.IdxCasl];
            };

            if (myCell.SR.Casr_Min >= myCell.TVc[Pd.IdxCasr])
            {
                myCell.SR.Casr_Min = myCell.TVc[Pd.IdxCasr];
            };
            if (myCell.SR.Casr_Max <= myCell.TVc[Pd.IdxCasr])
            {
                myCell.SR.Casr_Max = myCell.TVc[Pd.IdxCasr];
            };

            if (myCell.Mitochondria.Camit_Min >= myCell.TVc[Pd.InxCamit])
            {
                myCell.Mitochondria.Camit_Min = myCell.TVc[Pd.InxCamit];
            };
            if (myCell.Mitochondria.Camit_Max <= myCell.TVc[Pd.InxCamit])
            {
                myCell.Mitochondria.Camit_Max = myCell.TVc[Pd.InxCamit];
            };

            //Find dVdT peak
            double Threshold = 3.0;
            dVdTVal = (myCell.TVc[Pd.IdxVm] - VmOld) / dt;
            VmOld = Cell.Vm;

            if (PeakFlag == 0)
            {
                if (dVdTVal > Threshold)
                {
                    dVdTPeak[1].dVdTPeak = dVdTVal;
                    dVdTPeak[1].ElpTime = TimeElp;
                    dVdTPeak[1].SDTime = TimeElp_Sht;
                    PeakFlag = 1;
                }
            }
            else if (PeakFlag == 1)
            {
                if (dVdTVal > dVdTPeak[1].dVdTPeak)
                {
                    dVdTPeak[1].dVdTPeak = dVdTVal;
                    dVdTPeak[1].ElpTime = TimeElp;
                    dVdTPeak[1].SDTime = TimeElp_Sht;
                }
                else
                {
                    CL = dVdTPeak[1].ElpTime - dVdTPeak[0].ElpTime;
                    HR = 1000.0 / CL * 60.0;
                    lbl_HR.Text = String.Format("{0,3:00}", HR);
                    lbl_CL.Text = String.Format("{0,3:00}", CL);
                    lbl_HR.Refresh();
                    lbl_CL.Refresh();

                    Cell.TVc[Pd.InxHR] = HR;
                    Cell.TVc[Pd.InxCL] = CL;

                    dVdTPeak[0].dVdTPeak = dVdTPeak[1].dVdTPeak;
                    dVdTPeak[0].ElpTime = dVdTPeak[1].ElpTime;
                    dVdTPeak[0].SDTime = dVdTPeak[1].SDTime;
                    PeakFlag = 2;
                }
            }
            else if (PeakFlag == 2)
            {
                if (dVdTVal < Threshold)
                {
                    PeakFlag = 0;
                }
            }
        }

        private void DispMinMax(cCell myCell)
        {
            double Cai1000;
            double Cajs1000;
            double Casl1000;
            double Casr;
            double Camit1000;

            lbl_VmMin.Text = String.Format("{0,3:0.0}", myCell.Vm_Min);
            lbl_VmMax.Text = String.Format("{0,3:0.0}", myCell.Vm_Max);
            Cai1000 = myCell.Cyt.Cai_Min * 1000;
            lbl_CaiMin.Text = String.Format("{0,3:0.00}", Cai1000);
            Cai1000 = myCell.Cyt.Cai_Max * 1000;
            lbl_CaiMax.Text = String.Format("{0,3:0.00}", Cai1000);
            Cajs1000 = myCell.JS.Cajs_Min * 1000;
            lbl_CajsMin.Text = String.Format("{0,3:0.00}", Cajs1000);
            Cajs1000 = myCell.JS.Cajs_Max * 1000;
            lbl_CajsMax.Text = String.Format("{0,3:0.00}", Cajs1000);
            Casl1000 = myCell.SL.Casl_Min * 1000;
            lbl_CaslMin.Text = String.Format("{0,3:0.00}", Casl1000);
            Casl1000 = myCell.SL.Casl_Max * 1000;
            lbl_CaslMax.Text = String.Format("{0,3:0.00}", Casl1000);
            Casr = myCell.SR.Casr_Min ;
            lbl_CasrMin.Text = String.Format("{0,3:0.00}", Casr);
            Casr = myCell.SR.Casr_Max ;
            lbl_CasrMax.Text = String.Format("{0,3:0.00}", Casr);
            Camit1000 = myCell.Mitochondria.Camit_Min * 1000;
            lbl_CamitMin.Text = String.Format("{0,3:0.00}", Camit1000);
            Camit1000 = myCell.Mitochondria.Camit_Max * 1000;
            lbl_CamitMax.Text = String.Format("{0,3:0.00}", Camit1000);

            myCell.TVc[Pd.InxVm_Min] = myCell.Vm_Min;
            myCell.TVc[Pd.InxVm_Max] = myCell.Vm_Max;
            myCell.TVc[Pd.InxCai_Min] = myCell.Cyt.Cai_Min;
            myCell.TVc[Pd.InxCai_Max] = myCell.Cyt.Cai_Max;
            myCell.TVc[Pd.InxCajs_Min] = myCell.JS.Cajs_Min;
            myCell.TVc[Pd.InxCajs_Max] = myCell.JS.Cajs_Max;
            myCell.TVc[Pd.InxCasl_Min] = myCell.SL.Casl_Min;
            myCell.TVc[Pd.InxCasl_Max] = myCell.SL.Casl_Max;
            myCell.TVc[Pd.InxCasr_Min] = myCell.SR.Casr_Min;
            myCell.TVc[Pd.InxCasr_Max] = myCell.SR.Casr_Max;
            myCell.TVc[Pd.InxCamit_Min] = myCell.Mitochondria.Camit_Min;
            myCell.TVc[Pd.InxCamit_Max] = myCell.Mitochondria.Camit_Max;
        }

        private void SaveSimData(string strFileName, double ElapsedTime, int SamplCount)
        {
            int i = -1;
            if (SamplCount < Pd.mTVMax)
            {
                InputmTV(++i, SamplCount, ElapsedTime / 1000, "Time(sec)");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.IdxVm], "Vm");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.IdxNai], "Nai");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.IdxKi], "Ki");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.IdxCai], "Cai");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.IdxCli], "Cli");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.IdxCajs], "Cajs");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.IdxCasl], "Casl");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.IdxCasr], "Casr");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.IdxCamit_tot], "Camit_tot");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxCamit], "Camit");

                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxINajs_Itc], "INajs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxINasl_Itc], "INasl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxINaKjs_Itc], "INaKjs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxINaKsl_Itc], "INaKsl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIKrjs_Itc], "IKrjs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIKrsl_Itc], "IKrsl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIKsjs_Itc], "IKsjs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIKssl_Itc], "IKssl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIKpjs_Itc], "IKpjs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIKpsl_Itc], "IKpsl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIClCajs_Itc], "IClCajs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIClCasl_Itc], "IClCasl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICaLjs_Itc], "ICaLjs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICaL_Cajs], "ICaL_Cajs");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICaL_Najs], "ICaL_Najs");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICaL_Kjs], "ICaL_Kjs");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICaLsl_Itc], "ICaLsl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICaL_Casl], "ICaL_Casl");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICaL_Nasl], "ICaL_Nasl");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICaL_Ksl], "ICaL_Ksl");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxINaCajs_Itc], "INaCajs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxINaCasl_Itc], "INaCasl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIpCajs_Itc], "IpCajs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIpCasl_Itc], "IpCasl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICabjs_Itc], "ICabjs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICabsl_Itc], "ICabsl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIhajs_Itc], "Ihajs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIha_Najs], "Iha_Najs");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIha_Kjs], "Iha_Kjs");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIhasl_Itc], "Ihasl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIha_Nasl], "Iha_Nasl");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIha_Ksl], "Iha_Ksl");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICaTjs_Itc], "ICaTjs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxICaTsl_Itc], "ICaTsl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxItojs_Itc], "Itojs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxItosl_Itc], "Itosl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIKurjs_Itc], "IKurjs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIKursl_Itc], "IKursl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIK1js_Itc], "IK1js_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIK1sl_Itc], "IK1sl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIClbjs_Itc], "IClbjs_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxIClbsl_Itc], "IClbsl_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJCabuff_cyt_tot], "JCabuff_cyt_tot");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJCabuff_js_tot], "JCabuff_js_tot");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJCabuff_sl_tot], "JCabuff_sl_tot");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJCadiff_cytsl], "JCadiff_cytsl");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJCadiff_jssl], "JCadiff_jssl");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxSERCA_Itc], "SERCA_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJSERCA], "JSERCA");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxRyR_Itc], "RyR_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJRyR], "JRyR");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxLeak_Itc], "Leak_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJLeak], "JLeak");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJCSQN], "JCSQN");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxCaUni_Itc], "CaUni_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJCaUni], "JCaUni");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxNCXmit_Itc], "NCXmit_Itc");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxJNCXmit], "JNCXmit");

                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxHR], "HR");
                InputmTV(++i, SamplCount, Cell.TVc[Pd.InxCL], "CL");

                Pd.mTVIdxLast = i;
            };
        }

        private void SaveSimDataByBeat(string strFileName, double ElapsedTime, int SamplCount)
        {
            int i = -1;
            if (SamplCount < Pd.mTVMax_beat)
            {
                InputmTVbeat(++i, SamplCount, ElapsedTime / 1000, "Time(sec)");
                InputmTVbeat(++i, SamplCount, Cell.TVc[Pd.IdxNai], "Nai");
                InputmTVbeat(++i, SamplCount, Cell.TVc[Pd.IdxKi], "Ki");
                InputmTVbeat(++i, SamplCount, Cell.Vm_Min, "Vm_Min");
                InputmTVbeat(++i, SamplCount, Cell.Vm_Max, "Vm_Max");
                InputmTVbeat(++i, SamplCount, Cell.Cyt.Cai_Min, "Cai_Min");
                InputmTVbeat(++i, SamplCount, Cell.Cyt.Cai_Max, "Cai_Max");
                InputmTVbeat(++i, SamplCount, Cell.JS.Cajs_Min, "Cajs_Min");
                InputmTVbeat(++i, SamplCount, Cell.JS.Cajs_Max, "Cajs_Max");
                InputmTVbeat(++i, SamplCount, Cell.SL.Casl_Min, "Casl_Min");
                InputmTVbeat(++i, SamplCount, Cell.SL.Casl_Max, "Casl_Max");
                InputmTVbeat(++i, SamplCount, Cell.SR.Casr_Min, "Casr_Min");
                InputmTVbeat(++i, SamplCount, Cell.SR.Casr_Max, "Casr_Max");
                InputmTVbeat(++i, SamplCount, Cell.Mitochondria.Camit_Min, "Camit_Min");
                InputmTVbeat(++i, SamplCount, Cell.Mitochondria.Camit_Max, "Camit_Max");

                InputmTVbeat(++i, SamplCount, HR, "HR");
                InputmTVbeat(++i, SamplCount, CL, "CL");

                Pd.mTVIdxLast_beat = i;  // i - 1;      //  Must Be equal to or smaller than mTVIdxMax
            };
        }

        private void WriteSimData(string strFileName, double ElapsedTime, int SamplCount)
        {
            string Strtemp = "";
            System.IO.StreamWriter sw1 = new System.IO.StreamWriter(strFileName, false, Encoding.GetEncoding("Shift_JIS"));

            int j = -1;
            for (j = 0; j < Pd.mTVIdxLast; j++)
            {
                Strtemp = Strtemp + Cell.mTV[j].Str + ",";
            }
            j = Pd.mTVIdxLast;
            Strtemp = Strtemp + Cell.mTV[j].Str;
            sw1.WriteLine(Strtemp);

            Strtemp = "";
            int i;
            for (i = 0; i < (SamplCount - 1); i++)
            {
                Strtemp = "";
                for (j = 0; j < Pd.mTVIdxLast; j++)
                {
                    Strtemp = Strtemp + Cell.mTV[j].Val[i].ToString() + ",";
                    Cell.mTV[j].Val[i] = 0.0;       //0で再初期化

                }
                j = Pd.mTVIdxLast;
                Strtemp = Strtemp + Cell.mTV[j].Val[i].ToString();
                Cell.mTV[j].Val[i] = 0.0;       //0で再初期化
                sw1.WriteLine(Strtemp);
            }
            sw1.Close();
        }

        private void WriteSimDataBybeat(string strFileName, double ElapsedTime, int SamplCount)
        {
            string Strtemp = "";
            System.IO.StreamWriter sw1 = new System.IO.StreamWriter(strFileName, false, Encoding.GetEncoding("Shift_JIS"));

            int j = -1;
            for (j = 0; j < Pd.mTVIdxLast_beat; j++)
            {
                Strtemp = Strtemp + Cell.mTVbeat[j].Str + ",";
            }
            j = Pd.mTVIdxLast_beat;
            Strtemp = Strtemp + Cell.mTVbeat[j].Str;
            sw1.WriteLine(Strtemp);

            Strtemp = "";
            int i;
            for (i = 0; i < (SamplCount - 1); i++)
            {
                Strtemp = "";
                for (j = 0; j < Pd.mTVIdxLast_beat; j++)
                {
                    Strtemp = Strtemp + Cell.mTVbeat[j].Val[i].ToString() + ",";
                    Cell.mTVbeat[j].Val[i] = 0.0;       //0で再初期化

                }
                j = Pd.mTVIdxLast_beat;
                Strtemp = Strtemp + Cell.mTVbeat[j].Val[i].ToString();
                Cell.mTVbeat[j].Val[i] = 0.0;       //0で再初期化
                sw1.WriteLine(Strtemp);
            }
            sw1.Close();
        }

        private void InputmTV(int i, int SampCount, double Value, string ValStr)
        {
            if (SampCount == 0)
            {
                Cell.mTV[i].Str = ValStr;
                Cell.mTV[i].Val[SampCount] = Value;
            }
            else
            {
                Cell.mTV[i].Str = ValStr;
                Cell.mTV[i].Val[SampCount] = Value;
            }
        }

        private void InputmTVbeat(int i, int SampCount, double Value, string ValStr)
        {
            if (SampCount == 0)
            {
                Cell.mTVbeat[i].Str = ValStr;
                Cell.mTVbeat[i].Val[SampCount] = Value;
            }
            else
            {
                Cell.mTVbeat[i].Str = ValStr;
                Cell.mTVbeat[i].Val[SampCount] = Value;
            }
        }

        private void ReadInitVals()
        {
            System.IO.StreamReader sr = new System.IO.StreamReader(InitFileName, Encoding.GetEncoding("Shift_JIS"));
            int j = -1;
            while (sr.EndOfStream == false)
            {
                j = j + 1;
                if (sr.EndOfStream == true) break;
                string line = sr.ReadLine();
                string[] fields = line.Split(',');

                for (int i = 0; i < fields.Length; i++)
                {
                    if ((i == 1) && (j < Cell.TVc.Length))
                    {
                        Cell.TVc[j] = Convert.ToDouble(fields[i]);
                    }
                }
            }
            sr.Close();
        }

        private void SaveInitVals()
        {
            System.IO.StreamWriter writer = new System.IO.StreamWriter(InitFileName, false, Encoding.GetEncoding("Shift_JIS"));
            int j = -1;
            for (j = 0; j < Pd.IdxLast; j++)
            {
                writer.WriteLine(Cell.TVcStr[j] + "," + Cell.TVc[j]);
            }
            writer.Close();
        }

        private void ResetVariables()
        {
            ReadInitVals();

            TimeElp = 0.0;
            TimeElp_Sht = 0.0;
            TimeElp_Lng = 0.0;
            ElapsedTime_cyc = 0.0;

            lbl_Time.Text = "0.0";
            lblTime_min.Text = "0.0";

            lbl_Time.Refresh();
            lblTime_min.Refresh();

            Pd.BlnFirstLoad = true;
            Cell.InitializeCell(ref Cell, ref ListForm);
            Pd.BlnFirstLoad = false;

            DispForm1.EraseGraphs();
            DispForm2.EraseGraphs();
            DispForm3.EraseGraphs();
            DispForm4.EraseGraphs();

            ResetToolStripMenuItem.Enabled = true;
            StartToolStripMenuItem.Enabled = true;
            StopToolStripMenuItem.Enabled = true;
            SaveToolStripMenuItem.Enabled = true;
        }

        private void ResetNumericvaluesInTabs()
        {
            Txt_OneCycle.Text = "1000";
            Txt_SimeTimeMax.Text = "60";
            ComboBox1.Text = "s";
            //Txt_ShtDispTimeMax.Text = "1000";
            Txt_LngDispTimeMax.Text = "2000";
            ComboBox2.Text = "ms";
            Txt_PlotIntv.Text = "1.0";
            Txt_SimSaveIntV.Text = "1.0";
        }

        private int Roundoff(double dValue)
        {
            return (int)Math.Round(dValue, MidpointRounding.AwayFromZero);
        }

        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string FNametemp;
            OpenFileDialog sfd = new OpenFileDialog();
            string strCurrentDir = System.Environment.CurrentDirectory;

            Pd.NameTV(Cell.TVcStr);

            if (Rbtn_DefaultInitFile.Checked == true)
            {
                InitFileName = strCurrentDir + '\\' + "InitFiles" + '\\' + "HL1.asc";
                ReadInitVals();
            }
            else if (Rbtn_OtherInitFile.Checked)
            {
                //はじめのファイル名を指定する
                sfd.FileName = "HL1.asc";

                //はじめに表示されるフォルダを指定する
                //sfd.InitialDirectory = "C:\"
                //[ファイルの種類]に表示される選択肢を指定する
                sfd.Filter = "Initファイル(*.asc)|*.asc|すべてのファイル(*.*)|*.*";

                //[ファイルの種類]ではじめに
                //「すべてのファイル」が選択されているようにする
                sfd.FilterIndex = 1;

                //タイトルを設定する
                sfd.Title = "初期値ファイルを選択してください";

                //ダイアログボックスを閉じる前に現在のディレクトリを復元するようにする
                sfd.RestoreDirectory = true;

                //存在しないパスが指定されたとき警告を表示する
                //デフォルトでTrueなので指定する必要はない
                sfd.CheckPathExists = true;

                if (sfd.ShowDialog() == DialogResult.OK) ReadInitVals();
                FNametemp = System.IO.Path.GetFileName(sfd.FileName);
                InitFileName = sfd.FileName;
            };

            TimeElp = 0.0;
            TimeElp_Sht = 0.0;
            TimeElp_Lng = 0.0;
            ElapsedTime_cyc = 0.0;

            lbl_Time.Text = "0.0";
            lblTime_min.Text = "0.0";

            lbl_Time.Refresh();
            lblTime_min.Refresh();

            Pd.BlnFirstLoad = true;
            Cell.InitializeCell(ref Cell, ref ListForm);
            Cell.DispValues(ref ListForm);
            Pd.BlnFirstLoad = false;

            DispForm1.EraseGraphs();
            DispForm2.EraseGraphs();
            DispForm3.EraseGraphs();
            DispForm4.EraseGraphs();

            ResetToolStripMenuItem.Enabled = true;
            StartToolStripMenuItem.Enabled = true;
            StopToolStripMenuItem.Enabled = true;
            SaveToolStripMenuItem.Enabled = true;
        }

        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();

            //はじめのファイル名を指定する
            sfd.FileName = "HL1.asc";

            //はじめに表示されるフォルダを指定する
            //sfd.InitialDirectory = "C:\"
            //[ファイルの種類]に表示される選択肢を指定する
            sfd.Filter = "csvファイル(*.asc)|*.asc|すべてのファイル(*.*)|*.*";

            //[ファイルの種類]ではじめに
            //「すべてのファイル」が選択されているようにする
            sfd.FilterIndex = 1;

            //タイトルを設定する
            sfd.Title = "保存先のファイルを選択してください";

            //ダイアログボックスを閉じる前に現在のディレクトリを復元するようにする
            sfd.RestoreDirectory = true;

            //既に存在するファイル名を指定したとき警告する
            //デフォルトでTrueなので指定する必要はない
            sfd.OverwritePrompt = true;

            //存在しないパスが指定されたとき警告を表示する
            //デフォルトでTrueなので指定する必要はない
            sfd.CheckPathExists = true;

            //ダイアログを表示する
            if (sfd.ShowDialog() == DialogResult.OK) SaveInitVals();
            //OKボタンがクリックされたとき
            //選択されたファイル名を表示する 
        }

        private void QuitToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            ListForm.Hide();
            Close();
        }

        private void AllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ResetVariables();
            ResetNumericvaluesInTabs();
        }

        private void OnlyVariablesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ResetVariables();
        }

        private void TabSettingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ResetNumericvaluesInTabs();
        }

        private void StartToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ResetToolStripMenuItem.Enabled = false;
            StartToolStripMenuItem.Enabled = false;
            StopToolStripMenuItem.Enabled = true;
            FileToolStripMenuItem.Enabled = false;

            SimCells();

            ResetToolStripMenuItem.Enabled = true;
            StartToolStripMenuItem.Enabled = true;
            StopToolStripMenuItem.Enabled = false;
            FileToolStripMenuItem.Enabled = true;
        }

        private void StopToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BlnStopFlag = true;
            ResetToolStripMenuItem.Enabled = true;
            StartToolStripMenuItem.Enabled = true;
            StopToolStripMenuItem.Enabled = false;
            FileToolStripMenuItem.Enabled = true;
        }

        private void EraseSubDispsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DispForm1.EraseGraphs();
            DispForm2.EraseGraphs();
            DispForm3.EraseGraphs();
            DispForm4.EraseGraphs();
        }

        private void WiderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            double K = 1.1;
            DispForm1.Width = Roundoff(DispForm1.Width * K);
            DispForm2.Width = Roundoff(DispForm2.Width * K);
            DispForm3.Width = Roundoff(DispForm3.Width * K);
            DispForm4.Width = Roundoff(DispForm4.Width * K);
            AlignToolStripMenuItem_Click(this, e);
        }

        private void NarrowerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            double K = 0.9;
            DispForm1.Width = Roundoff(DispForm1.Width * K);
            DispForm2.Width = Roundoff(DispForm2.Width * K);
            DispForm3.Width = Roundoff(DispForm3.Width * K);
            DispForm4.Width = Roundoff(DispForm4.Width * K);
            AlignToolStripMenuItem_Click(this, e);
        }

        private void HigherToolStripMenuItem_Click(object sender, EventArgs e)
        {
            double K = 1.1;
            DispForm1.Height = Roundoff(DispForm1.Height * K);
            DispForm2.Height = Roundoff(DispForm2.Height * K);
            DispForm3.Height = Roundoff(DispForm3.Height * K);
            DispForm4.Height = Roundoff(DispForm4.Height * K);
            AlignToolStripMenuItem_Click(this, e);
        }

        private void LowerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            double K = 0.9;
            DispForm1.Height = Roundoff(DispForm1.Height * K);
            DispForm2.Height = Roundoff(DispForm2.Height * K);
            DispForm3.Height = Roundoff(DispForm3.Height * K);
            DispForm4.Height = Roundoff(DispForm4.Height * K);
            AlignToolStripMenuItem_Click(this, e);
        }

        private void AlignToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DispForm1.Top = 0;
            DispForm2.Top = 0;
            DispForm3.Top = 0;
            DispForm4.Top = 0;
            ListForm.Top = 0;

            DispForm1.Left = 0;
            DispForm2.Left = DispForm1.Width;
            DispForm3.Left = DispForm1.Width + DispForm2.Width;
            DispForm4.Left = DispForm1.Width + DispForm2.Width + DispForm3.Width;
            ListForm.Left = DispForm1.Width + DispForm2.Width + DispForm3.Width + DispForm4.Width;
        }

        public static System.Windows.Forms.MdiClient
        GetMdiClient(System.Windows.Forms.Form f)
        {
            foreach (System.Windows.Forms.Control c in f.Controls)
                if (c is System.Windows.Forms.MdiClient)
                    return (System.Windows.Forms.MdiClient)c;
            return null;
        }

        private void Btn_RunProtocol_Click(object sender, EventArgs e)
        {
            string Param1Str;
            string Param2Str;

            int I;
            int J;

            DateTime dtNow = DateTime.Now;
            string stDateTime = dtNow.ToString("yyyyMMdd_hhmmss");
            bool BlnFirstRun;

            BlnProtocolOn = true;

            Txt_SimeTimeMax.Text = "2"; // sec

            if (Chk_DisplayOnProtocol.Checked == true)
            {
                Cbx_ShowDispForms.Checked = true;
            }
            else
            {
                Cbx_ShowDispForms.Checked = false;
            }
            if (Chk_SaveLastCycle.Checked == true)
            {
                Rbn_SaveLC.Checked = true;
            }

            //******************** One parameter*******************************************************************************************************************
            //'***** Set Parameter 1**********
            double[] Param1Set = new double[21] { 0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9, 2.0 };

            Param1Str = "NCXmit.SF";
            Param2Str = "";
            //*******************************
            lbl_Param1.Text = Param1Str;

            lbl_Param1.Refresh();

            BlnFirstRun = true;

            // カレントディレクトリの取得
            string stCurrentDir = System.Environment.CurrentDirectory;
            string stInitFileDir = System.IO.Directory.GetParent(stCurrentDir).ToString();  // 一つ上
            stInitFileDir = System.IO.Directory.GetParent(stInitFileDir).ToString();　// もう一つ上

            // フォルダ (ディレクトリ) が存在しているかどうか確認する
            string DirectTemp = stInitFileDir + '\\' + "SimData";

            if (System.IO.Directory.Exists(DirectTemp))
            {
               // MessageBox.Show("ディレクトリは存在します");
            }
            else
            {
               // MessageBox.Show("ディレクトリは存在しません");
                System.IO.DirectoryInfo di = System.IO.Directory.CreateDirectory(DirectTemp);
            }

            for (J = 0; J < Param1Set.Length; ++J)
            {
                LoadToolStripMenuItem.PerformClick();
                if (Chk_SaveLastCycle.Checked == true)
                {
                    string Param1SetStr = Param1Set[J].ToString();
                    LCFNameForProtocol = stInitFileDir + '\\' + "SimData" + "\\" + stDateTime + "_" + Param1Str + Param1SetStr + "_LC.csv";
                    SimFileName = stInitFileDir + '\\' + "SimData" + "\\" + stDateTime + "_" + Param1Str + ".csv";
                    SimFileName_beat = stInitFileDir + '\\' + "SimData" + "\\" + stDateTime + "_" + Param1Str + Param1SetStr + "_beat.csv";

                    lbl_Param1_Value.Text = Param1SetStr;
                    lbl_Param1_Value.Refresh();

                    //****************************************************
                    Cell.Mitochondria.NCXmit.SF = Param1Set[J];
                    SimCells();
                    //****************************************************

                    if (Chk_SaveLastValues.Checked == true)
                    {
                        AutoSaveInitVals2Par(BlnFirstRun, SimFileName, Param1Str, Param1Set[J], "None", 0.0);
                    }

                    Application.DoEvents();
                    BlnFirstRun = false;
                }
            }
            //******************** One parameter end*****************************************************************



            ////******************** Two parameters *****************************************************************
            ////'***** Set Parameter 1**********
            //double[] Param1Set = new double[21] { 0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9, 2.0 };
            //double[] Param2Set = new double[21] { 0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9, 2.0 };

            //Param1Str = "NCXmit.SF";
            //Param2Str = "CaUni.SF";
            //////'*******************************
            //lbl_Param1.Text = Param1Str;
            //lbl_Param2.Text = Param2Str;
            //lbl_Param1.Refresh();

            //BlnFirstRun = true;

            //// カレントディレクトリの取得
            //string stCurrentDir = System.Environment.CurrentDirectory;
            //string stInitFileDir = System.IO.Directory.GetParent(stCurrentDir).ToString();  // 一つ上
            //stInitFileDir = System.IO.Directory.GetParent(stInitFileDir).ToString();　// もう一つ上

            //// フォルダ (ディレクトリ) が存在しているかどうか確認する
            //string DirectTemp = stInitFileDir + '\\' + "SimData";

            //if (System.IO.Directory.Exists(DirectTemp))
            //{
            //    // MessageBox.Show("ディレクトリは存在します");
            //}
            //else
            //{
            //    // MessageBox.Show("ディレクトリは存在しません");
            //    System.IO.DirectoryInfo di = System.IO.Directory.CreateDirectory(DirectTemp);
            //}

            //BlnProtocolOn = true;

            //for (I = 0; I < Param2Set.Length; ++I)
            //{
            //    for (J = 0; J < Param1Set.Length; ++J)
            //    {
            //        LoadToolStripMenuItem.PerformClick();
            //        if (Chk_SaveLastCycle.Checked == true)
            //        {
            //            string Param2SetStr = Param2Set[I].ToString();
            //            string Param1SetStr = Param1Set[J].ToString();
            //            LCFNameForProtocol = stInitFileDir + '\\' + "SimData" + "\\" + stDateTime + "_" + Param2Str + Param2SetStr + Param1Str + Param1SetStr + "_LC.csv";
            //            SimFileName = stInitFileDir + '\\' + "SimData" + "\\" + stDateTime + "_" + Param2Str + Param1Str + ".csv";
            //            SimFileName_beat = stInitFileDir + '\\' + "SimData" + "\\" + stDateTime + "_" + Param2Str + Param2SetStr + Param1Str + Param1SetStr + "_beat.csv";

            //            lbl_Param1_Value.Text = Param1SetStr;
            //            lbl_Param2_Value.Text = Param2SetStr;
            //            lbl_Param1_Value.Refresh();
            //            lbl_Param2_Value.Refresh();

            //            //'****************************************************
            //            Cell.Mitochondria.NCXmit.SF = Param1Set[J];
            //            Cell.Mitochondria.CaUni.SF = Param2Set[I];

            //            SimCells();
            //            //****************************************************

            //            if (Chk_SaveLastValues.Checked == true)
            //            {
            //                AutoSaveInitVals2Par(BlnFirstRun, SimFileName, Param2Str, Param2Set[I], Param1Str, Param1Set[J]);
            //            }

            //            Application.DoEvents();
            //            BlnFirstRun = false;

            //        }
            //    }
            //}
            ////******************** Two parameters end*****************************************************************

            BlnProtocolOn = false;
        }

        private void AutoSaveInitVals2Par(bool BlnFirstRun, string FName, string Param1Str, double Param1, string Param2Str, double Param2)
        {
            string strtemp;
            int K;

            // utf-8で書き込む
            // 書き込むファイルが既に存在している場合は、ファイルの末尾に追加する
            // 最後の値をセーブ
            System.IO.StreamWriter sw = new System.IO.StreamWriter(FName, true, Encoding.GetEncoding("utf-8"));

            if (BlnFirstRun == true)
            {
                strtemp = Param2Str + "," + Param1Str;
                for (K = 0; K < Cell.TVc.Length; ++K)
                {
                    if (Cell.TVcStr[K] != "")
                    {
                        strtemp = strtemp + "," + Cell.TVcStr[K];
                    }
                }
                sw.WriteLine(strtemp);
            }

            strtemp = Param2.ToString() + "," + Param1.ToString();

            for (K = 0; K < Cell.TVc.Length; ++K)
            {
                strtemp = strtemp + "," + Cell.TVc[K].ToString();
            }

            sw.WriteLine(strtemp);
            sw.Close();
        }

        private void HelpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox1.ShowDialog();
        }
    }
}

