﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cCaUni : cElement
    {
        private Pd.TIdx Ixalphamit;
        private Pd.TIdx Ixalphacyt;
        private Pd.TIdx IxACaUni;
        private Pd.TIdx IxJCaUni;

        public double alphamit = 0.2;
        public double alphacyt = 0.341;
        public double ACaUni = 23.2; // L/ms
        public double JCaUni;

        public cCaUni()  //コンストラクター 
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");
            SetIx(ref Ixalphamit, ref i, Pd.IntPar, "alphamit");
            SetIx(ref Ixalphacyt, ref i, Pd.IntPar, "alphacyt");
            SetIx(ref IxACaUni, ref i, Pd.IntPar, "ACaUni");
            SetIx(ref IxJCaUni, ref i, Pd.IntVar, "JCaUni");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpCaUni_ListView, NumOfIx);

            Itc = myTVc[Pd.InxCaUni_Itc];
            JCaUni = myTVc[Pd.InxJCaUni];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            double dPsi = myCell.Mitochondria.dPsi;
            double Cai = tvY[Pd.IdxCai];
            double Camit = myCell.Mitochondria.Camit;
            double Vmit = myCell.Mitochondria.Vmit;
            double zFVoRT = 2 * dPsi / Pd.RTF;
            double ezFVoRT = Math.Exp(-zFVoRT);
            double denomi = (ezFVoRT - 1.0);
            double nume = alphamit * Camit - alphacyt * Cai * ezFVoRT;

            if (denomi == 0.0)
            {
                Itc = SF * (Pd.F * 2 / myCell.Cm) * (-ACaUni);
                JCaUni = (myCell.Cm / (Vmit * Pd.F * 2)) * Itc;
            }
            else
            {
                Itc = SF * (Pd.F * 2 / myCell.Cm) * (-ACaUni) * zFVoRT * nume / denomi;
                JCaUni = (myCell.Cm / (Vmit * Pd.F * 2)) * Itc;
            }
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            myTVc[Pd.InxCaUni_Itc] = Itc;
            myTVc[Pd.InxJCaUni] = JCaUni;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpCaUni_ListView;
            ListView.LVDispValue("CaUni", IxSF, ref SF);
            ListView.LVDispValue("CaUni", IxItc, ref Itc);
            ListView.LVDispValue("CaUni", Ixalphamit, ref alphamit);
            ListView.LVDispValue("CaUni", Ixalphacyt, ref alphacyt);
            ListView.LVDispValue("CaUni", IxACaUni, ref ACaUni);
            ListView.LVDispValue("CaUni", IxJCaUni, ref JCaUni);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpCaUni_ListView;
            ListView.LVModiValue("CaUni", IxSF, ref SF);
            ListView.LVModiValue("CaUni", IxItc, ref Itc);
            ListView.LVModiValue("CaUni", Ixalphamit, ref alphamit);
            ListView.LVModiValue("CaUni", Ixalphacyt, ref alphacyt);
            ListView.LVModiValue("CaUni", IxACaUni, ref ACaUni);
            ListView.LVModiValue("CaUni", IxJCaUni, ref JCaUni);
        }
    }
}
