﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cCell : cElement
    {
        private Pd.TIdx IxCm;
        private Pd.TIdx IxVcell;
        private Pd.TIdx IxVm;
        private Pd.TIdx IxVm_Max;
        private Pd.TIdx IxVm_Min;

        private Pd.TIdx IxINatot;
        private Pd.TIdx IxIKtot;
        private Pd.TIdx IxICatot;
        private Pd.TIdx IxICltot;

        public double Cm = 40.0;       // Cell capacitance (pF)
        public double Vcell = 33000;  // Cell volume (um^3)

        public double Vm;       // Membrane potential (mV)
        public double Vm_Max;       // Membrane potential (mV)
        public double Vm_Min;       // Membrane potential (mV)

        public double INatot;
        public double IKtot;
        public double ICatot;
        public double ICltot;

        public Pd.TIons Rp;

        public Pd.TModelSetting MSet = new Pd.TModelSetting();
        public cExtCell ExtCell = new cExtCell();
        public cSR SR = new cSR();
        public cMitochondria Mitochondria = new cMitochondria(); 
        public cJS JS = new cJS();
        public cSL SL = new cSL();
        public cCyt Cyt = new cCyt();

        public cCadiff_cytsl Cadiff_cytsl = new cCadiff_cytsl();
        public cCadiff_jssl Cadiff_jssl = new cCadiff_jssl();

        public cINajs INajs = new cINajs();
        public cINasl INasl = new cINasl();
        public cINaKjs INaKjs = new cINaKjs();
        public cINaKsl INaKsl = new cINaKsl();
        public cIKrjs IKrjs = new cIKrjs();
        public cIKrsl IKrsl = new cIKrsl();
        public cIKsjs IKsjs = new cIKsjs();
        public cIKssl IKssl = new cIKssl();
        public cIKpjs IKpjs = new cIKpjs();
        public cIKpsl IKpsl = new cIKpsl();
        public cIClCajs IClCajs = new cIClCajs();
        public cIClCasl IClCasl = new cIClCasl();
        public cICaLjs ICaLjs = new cICaLjs();
        public cICaLsl ICaLsl = new cICaLsl();
        public cINaCajs INaCajs = new cINaCajs();
        public cINaCasl INaCasl = new cINaCasl();
        public cIpCajs IpCajs = new cIpCajs();
        public cIpCasl IpCasl = new cIpCasl();
        public cICabjs ICabjs = new cICabjs();
        public cICabsl ICabsl = new cICabsl();
        public cIhajs Ihajs = new cIhajs();
        public cIhasl Ihasl = new cIhasl();
        public cICaTjs ICaTjs = new cICaTjs();
        public cICaTsl ICaTsl = new cICaTsl();
        public cItojs Itojs = new cItojs();
        public cItosl Itosl = new cItosl();
        public cIKurjs IKurjs = new cIKurjs();
        public cIKursl IKursl = new cIKursl();
        public cIK1js IK1js = new cIK1js();
        public cIK1sl IK1sl = new cIK1sl();
        public cIClbjs IClbjs = new cIClbjs();
        public cIClbsl IClbsl = new cIClbsl();

        public double[] TVc = new double[Pd.IdxLast];
        public string[] TVcStr = new string[Pd.IdxLast];
        public double[] tvDYdt = new double[Pd.IdxLast]; //dummy

        public Pd.mTV_VStr[] mTV;
        public Pd.mTV_VStr2[] mTVbeat;

        public cCell()  //コンストラクター 
        {
            mTV = new Pd.mTV_VStr[Pd.mTVIdxMax];
            mTVbeat = new Pd.mTV_VStr2[Pd.mTVMax_beat];

            for (int i = 0; i <= (Pd.mTVIdxMax - 1); i++)
            {
                mTV[i] = new Pd.mTV_VStr();
            };

            for (int i = 0; i <= (Pd.mTVMax_beat - 1); i++)
            {
                mTVbeat[i] = new Pd.mTV_VStr2();
            };
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            Vm = myTVc[Pd.IdxVm];

            int i = 0;

            SetIx(ref IxCm, ref i, Pd.IntPar, "Cm");
            SetIx(ref IxVcell, ref i, Pd.IntPar, "Vcell");
            SetIx(ref IxVm, ref i, Pd.IntVar, "Vm");
            SetIx(ref IxINatot, ref i, Pd.IntVar, "INatot");
            SetIx(ref IxIKtot, ref i, Pd.IntVar, "IKtot");
            SetIx(ref IxICatot, ref i, Pd.IntVar, "ICatot");
            SetIx(ref IxICltot, ref i, Pd.IntVar, "ICltot");
            SetIx(ref IxVm_Max, ref i, Pd.IntVar, "Vm_Max");
            SetIx(ref IxVm_Min, ref i, Pd.IntVar, "Vm_Min");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpCell_ListView, NumOfIx);
        }

        public void InitializeCell(ref cCell myCell, ref ListForm Lf)
        {
            Initialize(ref TVc, ref myCell, ref Lf);

            ExtCell.Initialize(ref TVc, ref myCell, ref Lf);
            SR.Initialize(ref TVc, ref myCell, ref Lf);
            Mitochondria.Initialize(ref TVc, ref myCell, ref Lf);
            JS.Initialize(ref TVc, ref myCell, ref Lf);
            SL.Initialize(ref TVc, ref myCell, ref Lf);
            Cyt.Initialize(ref TVc, ref myCell, ref Lf);

            Cadiff_cytsl.Initialize(ref TVc, ref myCell, ref Lf);
            Cadiff_jssl.Initialize(ref TVc, ref myCell, ref Lf);

            INajs.Initialize(ref TVc, ref myCell, ref Lf);
            INasl.Initialize(ref TVc, ref myCell, ref Lf);
            INaKjs.Initialize(ref TVc, ref myCell, ref Lf);
            INaKsl.Initialize(ref TVc, ref myCell, ref Lf);
            IKrjs.Initialize(ref TVc, ref myCell, ref Lf);
            IKrsl.Initialize(ref TVc, ref myCell, ref Lf);
            IKsjs.Initialize(ref TVc, ref myCell, ref Lf);
            IKssl.Initialize(ref TVc, ref myCell, ref Lf);
            IKpjs.Initialize(ref TVc, ref myCell, ref Lf);
            IKpsl.Initialize(ref TVc, ref myCell, ref Lf);
            IClCajs.Initialize(ref TVc, ref myCell, ref Lf);
            IClCasl.Initialize(ref TVc, ref myCell, ref Lf);
            ICaLjs.Initialize(ref TVc, ref myCell, ref Lf);
            ICaLsl.Initialize(ref TVc, ref myCell, ref Lf);
            INaCajs.Initialize(ref TVc, ref myCell, ref Lf);
            INaCasl.Initialize(ref TVc, ref myCell, ref Lf);
            IpCajs.Initialize(ref TVc, ref myCell, ref Lf);
            IpCasl.Initialize(ref TVc, ref myCell, ref Lf);
            ICabjs.Initialize(ref TVc, ref myCell, ref Lf);
            ICabsl.Initialize(ref TVc, ref myCell, ref Lf);
            Ihajs.Initialize(ref TVc, ref myCell, ref Lf);
            Ihasl.Initialize(ref TVc, ref myCell, ref Lf);
            ICaTjs.Initialize(ref TVc, ref myCell, ref Lf);
            ICaTsl.Initialize(ref TVc, ref myCell, ref Lf);
            Itojs.Initialize(ref TVc, ref myCell, ref Lf);
            Itosl.Initialize(ref TVc, ref myCell, ref Lf); 
            IKurjs.Initialize(ref TVc, ref myCell, ref Lf);
            IKursl.Initialize(ref TVc, ref myCell, ref Lf); 
            IK1js.Initialize(ref TVc, ref myCell, ref Lf);
            IK1sl.Initialize(ref TVc, ref myCell, ref Lf);
            IClbjs.Initialize(ref TVc, ref myCell, ref Lf);
            IClbsl.Initialize(ref TVc, ref myCell, ref Lf);
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            ExtCell.dydt(dt, ref tvDYdt, ref tvY, myCell);
            SR.dydt(dt, ref tvDYdt, ref tvY, myCell);
            Mitochondria.dydt(dt, ref tvDYdt, ref tvY, myCell);
            JS.dydt(dt, ref tvDYdt, ref tvY, myCell);
            SL.dydt(dt, ref tvDYdt, ref tvY, myCell);
            Cyt.dydt(dt, ref tvDYdt, ref tvY, myCell);

            Cadiff_cytsl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            Cadiff_jssl.dydt(dt, ref tvDYdt, ref tvY, myCell);

            INajs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            INasl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            INaKjs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            INaKsl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IKrjs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IKrsl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IKsjs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IKssl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IKpjs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IKpsl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IClCajs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IClCasl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            ICaLjs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            ICaLsl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            INaCajs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            INaCasl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IpCajs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IpCasl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            ICabjs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            ICabsl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            Ihajs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            Ihasl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            ICaTjs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            ICaTsl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            Itojs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            Itosl.dydt(dt, ref tvDYdt, ref tvY, myCell); 
            IKurjs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IKursl.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IK1js.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IK1sl.dydt(dt, ref tvDYdt, ref tvY, myCell); 
            IClbjs.dydt(dt, ref tvDYdt, ref tvY, myCell);
            IClbsl.dydt(dt, ref tvDYdt, ref tvY, myCell);

            INatot = myCell.INajs.Itc + myCell.INasl.Itc + 3 * myCell.INaKjs.Itc + 3 * myCell.INaKsl.Itc + 3 * myCell.INaCajs.Itc + 3 * myCell.INaCasl.Itc + myCell.ICaLjs.ICaL_Na + myCell.ICaLsl.ICaL_Na + myCell.Ihajs.Iha_Na + myCell.Ihasl.Iha_Na;
            IKtot = myCell.IKrjs.Itc + myCell.IKrsl.Itc + myCell.IKsjs.Itc + myCell.IKssl.Itc + myCell.IKpjs.Itc + myCell.IKpsl.Itc + myCell.Itojs.Itc + myCell.Itosl.Itc + myCell.IKurjs.Itc + myCell.IKursl.Itc + myCell.IK1js.Itc + myCell.IK1sl.Itc + myCell.ICaLjs.ICaL_K + myCell.ICaLsl.ICaL_K + myCell.Ihajs.Iha_K + myCell.Ihasl.Iha_K  - 2 * myCell.INaKjs.Itc - 2 * myCell.INaKsl.Itc;
            ICatot = myCell.ICaLjs.ICaL_Ca + myCell.ICaLsl.ICaL_Ca + myCell.ICaTjs.Itc + myCell.ICaTsl.Itc + myCell.IpCajs.Itc + myCell.IpCasl.Itc + myCell.ICabjs.Itc + myCell.ICabsl.Itc - 2 * myCell.INaCajs.Itc - 2 * myCell.INaCasl.Itc;
            ICltot = myCell.IClCajs.Itc + myCell.IClCasl.Itc + myCell.IClbjs.Itc + myCell.IClbsl.Itc;

            tvDYdt[Pd.IdxVm] = -(INatot + IKtot + ICatot + ICltot);
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            ExtCell.TransforToTV(ref myTVc, myCell);
            SR.TransforToTV(ref myTVc, myCell);
            Mitochondria.TransforToTV(ref myTVc, myCell); 
            JS.TransforToTV(ref myTVc, myCell);
            SL.TransforToTV(ref myTVc, myCell);
            Cyt.TransforToTV(ref myTVc, myCell);

            Cadiff_cytsl.TransforToTV(ref myTVc, myCell);
            Cadiff_jssl.TransforToTV(ref myTVc, myCell);

            INajs.TransforToTV(ref myTVc, myCell);
            INasl.TransforToTV(ref myTVc, myCell);
            INaKjs.TransforToTV(ref myTVc, myCell);
            INaKsl.TransforToTV(ref myTVc, myCell);
            IKrjs.TransforToTV(ref myTVc, myCell);
            IKrsl.TransforToTV(ref myTVc, myCell);
            IKsjs.TransforToTV(ref myTVc, myCell);
            IKssl.TransforToTV(ref myTVc, myCell);
            IKpjs.TransforToTV(ref myTVc, myCell);
            IKpsl.TransforToTV(ref myTVc, myCell);
            IClCajs.TransforToTV(ref myTVc, myCell);
            IClCasl.TransforToTV(ref myTVc, myCell);
            ICaLjs.TransforToTV(ref myTVc, myCell);
            ICaLsl.TransforToTV(ref myTVc, myCell);
            INaCajs.TransforToTV(ref myTVc, myCell);
            INaCasl.TransforToTV(ref myTVc, myCell);
            IpCajs.TransforToTV(ref myTVc, myCell);
            IpCasl.TransforToTV(ref myTVc, myCell);
            ICabjs.TransforToTV(ref myTVc, myCell);
            ICabsl.TransforToTV(ref myTVc, myCell);
            Ihajs.TransforToTV(ref myTVc, myCell);
            Ihasl.TransforToTV(ref myTVc, myCell);
            ICaTjs.TransforToTV(ref myTVc, myCell);
            ICaTsl.TransforToTV(ref myTVc, myCell);
            Itojs.TransforToTV(ref myTVc, myCell);
            Itosl.TransforToTV(ref myTVc, myCell);
            IKurjs.TransforToTV(ref myTVc, myCell);
            IKursl.TransforToTV(ref myTVc, myCell);
            IK1js.TransforToTV(ref myTVc, myCell);
            IK1sl.TransforToTV(ref myTVc, myCell);
            IClbjs.TransforToTV(ref myTVc, myCell);
            IClbsl.TransforToTV(ref myTVc, myCell);
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpCell_ListView;
            ListView.LVDispValue("Cell", IxCm, ref Cm);
            ListView.LVDispValue("Cell", IxVcell, ref Vcell);
            ListView.LVDispValue("Cell", IxVm, ref Vm);
            ListView.LVDispValue("Cell", IxINatot, ref INatot);
            ListView.LVDispValue("Cell", IxIKtot, ref IKtot);
            ListView.LVDispValue("Cell", IxICatot, ref ICatot);
            ListView.LVDispValue("Cell", IxICltot, ref ICltot);
            ListView.LVDispValue("Cell", IxVm_Max, ref Vm_Max);
            ListView.LVDispValue("Cell", IxVm_Min, ref Vm_Min);

            ExtCell.DispValues(ref Lf);
            SR.DispValues(ref Lf);
            Mitochondria.DispValues(ref Lf);
            JS.DispValues(ref Lf);
            SL.DispValues(ref Lf);
            Cyt.DispValues(ref Lf);

            Cadiff_cytsl.DispValues(ref Lf);
            Cadiff_jssl.DispValues(ref Lf);

            INajs.DispValues(ref Lf);
            INasl.DispValues(ref Lf);
            INaKjs.DispValues(ref Lf);
            INaKsl.DispValues(ref Lf);
            IKrjs.DispValues(ref Lf);
            IKrsl.DispValues(ref Lf);
            IKsjs.DispValues(ref Lf);
            IKssl.DispValues(ref Lf);
            IKpjs.DispValues(ref Lf);
            IKpsl.DispValues(ref Lf);
            IClCajs.DispValues(ref Lf);
            IClCasl.DispValues(ref Lf);
            ICaLjs.DispValues(ref Lf);
            ICaLsl.DispValues(ref Lf);
            INaCajs.DispValues(ref Lf);
            INaCasl.DispValues(ref Lf);
            IpCajs.DispValues(ref Lf);
            IpCasl.DispValues(ref Lf);
            ICabjs.DispValues(ref Lf);
            ICabsl.DispValues(ref Lf);
            Ihajs.DispValues(ref Lf);
            Ihasl.DispValues(ref Lf);
            ICaTjs.DispValues(ref Lf);
            ICaTsl.DispValues(ref Lf);
            Itojs.DispValues(ref Lf);
            Itosl.DispValues(ref Lf); 
            IKurjs.DispValues(ref Lf);
            IKursl.DispValues(ref Lf); 
            IK1js.DispValues(ref Lf);
            IK1sl.DispValues(ref Lf); 
            IClbjs.DispValues(ref Lf);
            IClbsl.DispValues(ref Lf);
        }

        public void ModiValues(ref ListForm Lf, cCell myCell)
        {
            ucListView ListView = Lf.tpCell_ListView;
            ListView.LVModiValue("Cell", IxCm, ref Cm);
            ListView.LVModiValue("Cell", IxVcell, ref Vcell);
            ListView.LVModiValue("Cell", IxVm, ref Vm);
            ListView.LVModiValue("Cell", IxINatot, ref INatot);
            ListView.LVModiValue("Cell", IxIKtot, ref IKtot);
            ListView.LVModiValue("Cell", IxICatot, ref ICatot);
            ListView.LVModiValue("Cell", IxICltot, ref ICltot);
            ListView.LVModiValue("Cell", IxVm_Max, ref Vm_Max);
            ListView.LVModiValue("Cell", IxVm_Min, ref Vm_Min);

            ExtCell.ModiValues(ref Lf);
            SR.ModiValues(ref Lf);
            Mitochondria.ModiValues(ref Lf); 
            JS.ModiValues(ref Lf);
            SL.ModiValues(ref Lf);
            Cyt.ModiValues(ref Lf);

            Cadiff_cytsl.ModiValues(ref Lf);
            Cadiff_jssl.ModiValues(ref Lf);

            INajs.ModiValues(ref Lf);
            INasl.ModiValues(ref Lf);
            INaKjs.ModiValues(ref Lf);
            INaKsl.ModiValues(ref Lf);
            IKrjs.ModiValues(ref Lf);
            IKrsl.ModiValues(ref Lf);
            IKsjs.ModiValues(ref Lf);
            IKssl.ModiValues(ref Lf);
            IKpjs.ModiValues(ref Lf);
            IKpsl.ModiValues(ref Lf);
            IClCajs.ModiValues(ref Lf);
            IClCasl.ModiValues(ref Lf);
            ICaLjs.ModiValues(ref Lf);
            ICaLsl.ModiValues(ref Lf);
            INaCajs.ModiValues(ref Lf);
            INaCasl.ModiValues(ref Lf);
            IpCajs.ModiValues(ref Lf);
            IpCasl.ModiValues(ref Lf);
            ICabjs.ModiValues(ref Lf);
            ICabsl.ModiValues(ref Lf);
            Ihajs.ModiValues(ref Lf);
            Ihasl.ModiValues(ref Lf);
            ICaTjs.ModiValues(ref Lf);
            ICaTsl.ModiValues(ref Lf);
            Itojs.ModiValues(ref Lf);
            Itosl.ModiValues(ref Lf); 
            IKurjs.ModiValues(ref Lf);
            IKursl.ModiValues(ref Lf); 
            IK1js.ModiValues(ref Lf);
            IK1sl.ModiValues(ref Lf);
            IClbjs.ModiValues(ref Lf);
            IClbsl.ModiValues(ref Lf);
        }

        protected void ConstantField(ref double[] tvaY)
        {
            double RTF = Pd.RTF;
            double RTF2 = Pd.RTF2;

            //--------------- reversal potential---------------------  
            Rp.Na = RTF * Math.Log(ExtCell.Nao / tvaY[Pd.IdxNai]);
            Rp.K = RTF * Math.Log(ExtCell.Ko / tvaY[Pd.IdxKi]);
            Rp.Cajs = RTF2 * Math.Log(ExtCell.Cao / tvaY[Pd.IdxCajs]);
            Rp.Casl = RTF2 * Math.Log(ExtCell.Cao / tvaY[Pd.IdxCasl]);
            Rp.Cl = RTF * Math.Log(tvaY[Pd.IdxCli] / ExtCell.Clo);

            tvaY[Pd.InxRpNa] = Rp.Na;
            tvaY[Pd.InxRpK] = Rp.K;
            tvaY[Pd.InxRpCajs] = Rp.Cajs;
            tvaY[Pd.InxRpCasl] = Rp.Casl;
            tvaY[Pd.InxRpCl] = Rp.Cl;
        }

        public void Integrate(ref double dt, ref double[] mytvY, cCell myCell)
        {
            ConstantField(ref mytvY);
            RungeKutta.rkqc(ref dt, ref mytvY, myCell);
            myCell.TransforToTV(ref mytvY, myCell);
        }
    }
}
