﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cCyt:cElement
    {
        public cCabuffcyt Cabuffcyt = new cCabuffcyt(); 
        
        private Pd.TIdx IxNai;
        private Pd.TIdx IxKi;
        private Pd.TIdx IxCai;
        private Pd.TIdx IxCli;
        private Pd.TIdx IxMgi;

        private Pd.TIdx IxVcyt_part;
        private Pd.TIdx IxVcyt;

        private Pd.TIdx IxCai_Min;
        private Pd.TIdx IxCai_Max;        

        public double Nai;  // intracellular Na concentration (mM)
        public double Ki;   // intracellular K concentration (mM)
        public double Cai;  // intracellular Ca concentration (mM)
        public double Cli;  // intracellular Cl concentration (mM)
        public double Mgi;  // intracellular Mg concentration (mM)

        public double Vcyt_part = 0.65; // part of cell volume occupied with cytoplasm
        public double Vcyt; // cytoplasm volume

        public double Cai_Min;
        public double Cai_Max;

        public cCyt()  //コンストラクター 
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            Nai = myTVc[Pd.IdxNai];
            Ki = myTVc[Pd.IdxKi];
            Cai = myTVc[Pd.IdxCai];
            Cli = myTVc[Pd.IdxCli];
            Mgi = 1.0; // mM

            Vcyt = Vcyt_part * myCell.Vcell;

            Cabuffcyt.Initialize(ref myTVc, ref myCell, ref Lf);

            int i = 0;

            SetIx(ref IxNai, ref i, Pd.IntVar, "Nai");
            SetIx(ref IxKi, ref i, Pd.IntVar, "Ki");
            SetIx(ref IxCai, ref i, Pd.IntVar, "Cai");
            SetIx(ref IxCli, ref i, Pd.IntVar, "Cli");
            SetIx(ref IxMgi, ref i, Pd.IntPar, "Mgi");
            SetIx(ref IxVcyt_part, ref i, Pd.IntPar, "Vcyt_part");
            SetIx(ref IxVcyt, ref i, Pd.IntVar, "Vcyt");
            SetIx(ref IxCai_Min, ref i, Pd.IntVar, "CaiMin");
            SetIx(ref IxCai_Max, ref i, Pd.IntVar, "CaiMax");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpCyt_ListView, NumOfIx);
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            Cabuffcyt.dydt(dt, ref tvDYdt, ref tvY, myCell);
            tvDYdt[Pd.IdxCai] = -(myCell.SR.SERCA.Itc - myCell.Mitochondria.CaUni.Itc + 2 * myCell.Mitochondria.NCXmit.Itc) * (myCell.Cm / (Vcyt * 2 * Pd.F)) - (tvDYdt[Pd.IdxbTnClow] + tvDYdt[Pd.IdxbTnChCa] + tvDYdt[Pd.IdxbTnChMg] + tvDYdt[Pd.IdxbCaM] + tvDYdt[Pd.IdxbMyocinCa] + tvDYdt[Pd.IdxbMyocinMg] + tvDYdt[Pd.IdxbSRB]) + myCell.Cadiff_cytsl.JCadif_cytsl / Vcyt;
            tvDYdt[Pd.IdxNai] = 0.0;
            tvDYdt[Pd.IdxKi] = 0.0;
            tvDYdt[Pd.IdxCli] = 0.0;
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            Cabuffcyt.TransforToTV(ref myTVc, myCell); 
            Cai = myTVc[Pd.IdxCai]; 
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpCyt_ListView;
            ListView.LVDispValue("Cyt", IxNai, ref Nai);
            ListView.LVDispValue("Cyt", IxKi, ref Ki);
            ListView.LVDispValue("Cyt", IxCai, ref Cai);
            ListView.LVDispValue("Cyt", IxCli, ref Cli);
            ListView.LVDispValue("Cyt", IxMgi, ref Mgi);

            ListView.LVDispValue("Cyt", IxVcyt_part, ref Vcyt_part);
            ListView.LVDispValue("Cyt", IxVcyt, ref Vcyt);

            ListView.LVDispValue("Cyt", IxCai_Min, ref Cai_Min);
            ListView.LVDispValue("Cyt", IxCai_Max, ref Cai_Max);

            Cabuffcyt.DispValues(ref Lf);
        }
        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpCyt_ListView;
            ListView.LVModiValue("Cyt", IxNai, ref Nai);
            ListView.LVModiValue("Cyt", IxKi, ref Ki);
            ListView.LVModiValue("Cyt", IxCai, ref Cai);
            ListView.LVModiValue("Cyt", IxCli, ref Cli);
            ListView.LVModiValue("Cyt", IxMgi, ref Mgi);

            ListView.LVModiValue("Cyt", IxVcyt_part, ref Vcyt_part);
            ListView.LVModiValue("Cyt", IxVcyt, ref Vcyt);

            ListView.LVModiValue("Cyt", IxCai_Min, ref Cai_Min);
            ListView.LVModiValue("Cyt", IxCai_Max, ref Cai_Max);

            Cabuffcyt.ModiValues(ref Lf);
        }
    }
}
