﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cICaLsl : cElement
    {
        private Pd.TIdx Ixd;
        private Pd.TIdx Ixd_inf;
        private Pd.TIdx Ixd_tau;

        private Pd.TIdx Ixf;
        private Pd.TIdx Ixf_inf;
        private Pd.TIdx Ixf_tau;

        private Pd.TIdx IxfCaB;

        private Pd.TIdx IxICaL_PCa;
        private Pd.TIdx IxICaL_PNa;
        private Pd.TIdx IxICaL_PK;

        private Pd.TIdx IxICaL_Ca;
        private Pd.TIdx IxICaL_Na;
        private Pd.TIdx IxICaL_K;

        public double d;
        public double d_inf;
        public double d_tau;

        public double f;
        public double f_inf;
        public double f_tau;

        public double fCaB;

        public double ICaL_PCa = 0.200475; //cm/sec
        public double ICaL_PNa = 5.56875e-6; //cm/sec
        public double ICaL_PK = 1.002375e-4; //cm/sec

        public double ICaL_Ca; 
        public double ICaL_Na; 
        public double ICaL_K; 

        public cICaLsl()  //コンストラクター
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");

            SetIx(ref Ixd, ref i, Pd.IntVar, "d");
            SetIx(ref Ixd_inf, ref i, Pd.IntVar, "d_inf");
            SetIx(ref Ixd_tau, ref i, Pd.IntVar, "d_tau");

            SetIx(ref Ixf, ref i, Pd.IntVar, "f");
            SetIx(ref Ixf_inf, ref i, Pd.IntVar, "f_inf");
            SetIx(ref Ixf_tau, ref i, Pd.IntVar, "f_tau");

            SetIx(ref IxfCaB, ref i, Pd.IntVar, "fCaB");

            SetIx(ref IxICaL_PCa, ref i, Pd.IntPar, "ICaL_PCa");
            SetIx(ref IxICaL_PNa, ref i, Pd.IntPar, "ICaL_PNa");
            SetIx(ref IxICaL_PK, ref i, Pd.IntPar, "ICaL_PK");

            SetIx(ref IxICaL_Ca, ref i, Pd.IntVar, "ICaL_Ca");
            SetIx(ref IxICaL_Na, ref i, Pd.IntVar, "ICaL_Na");
            SetIx(ref IxICaL_K, ref i, Pd.IntVar, "ICaL_K");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpICaLsl_ListView, NumOfIx);

            d = myTVc[Pd.IdxICaL_d_sl];
            f = myTVc[Pd.IdxICaL_f_sl];
            fCaB = myTVc[Pd.IdxICaL_fCaB_sl];
            Itc = myTVc[Pd.InxICaLsl_Itc];
            ICaL_Ca = myTVc[Pd.InxICaL_Casl];
            ICaL_Na = myTVc[Pd.InxICaL_Nasl];
            ICaL_K = myTVc[Pd.InxICaL_Ksl];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            double Vm = tvY[Pd.IdxVm];
            double VFRT = Vm / Pd.RTF;
            double Cao = myCell.ExtCell.Cao;
            double Cai = tvY[Pd.IdxCasl];
            double Ko = myCell.ExtCell.Ko;
            double Ki = tvY[Pd.IdxKi];
            double Nao = myCell.ExtCell.Nao;
            double Nai = tvY[Pd.IdxNai];
            double ISO = myCell.ExtCell.ISO;
            double Exp_VFRT = Math.Exp(VFRT);
            double Exp_2VFRT = Math.Exp(2*VFRT);

            double ICaL_PCa_ = ICaL_PCa * (1 + 0.5 * ISO * 1000);
            double ICaL_PNa_ = ICaL_PNa * (1 + 0.5 * ISO * 1000);
            double ICaL_PK_ = ICaL_PK * (1 + 0.5 * ISO * 1000);

            d_inf = 1.0 / (1.0 + Math.Exp((-1 - Vm - 3 * ISO * 1000) / 6.0));
            d_tau = d_inf * ((1 - Math.Exp((-5 - Vm - 3 * ISO * 1000) / 6.0)) / (0.035 * (Vm + 5 + 3 * ISO * 1000)));

            d = tvY[Pd.IdxICaL_d_sl];
            tvDYdt[Pd.IdxICaL_d_sl] = (d_inf - d) / d_tau;

            f_inf = 1.0 / (1.0 + Math.Exp((Vm + 30 + 3 * ISO * 1000) / 9)) + 0.6 / (1.0 + Math.Exp((50 - Vm - 3 * ISO * 1000) / 20));
            f_tau = 2 / (0.0197 * Math.Exp(-(0.0337 * (Vm + 14.5 + 3 * ISO * 1000)) * (0.0337 * (Vm + 14.5 + 3 * ISO * 1000))) + 0.02);

            f = tvY[Pd.IdxICaL_f_sl];
            tvDYdt[Pd.IdxICaL_f_sl] = (f_inf - f) / f_tau;

            fCaB = tvY[Pd.IdxICaL_fCaB_sl];
            tvDYdt[Pd.IdxICaL_fCaB_sl] = 1.7 * Cai * (1 - fCaB) - (11.9E-3) * fCaB;

            ICaL_Ca = SF * myCell.SL.fraction_slCa * ICaL_PCa_ * d * f * (1 - fCaB) * 4 * Pd.F * VFRT * (0.341 * Cai * Exp_2VFRT - 0.341 * Cao) / (Exp_2VFRT - 1.0);
            ICaL_Na = SF * myCell.SL.fraction_slCa * ICaL_PNa_ * d * f * (1 - fCaB) * Pd.F * VFRT * (0.75 * Nai * Exp_VFRT - 0.75 * Nao) / (Exp_VFRT - 1.0);
            ICaL_K = SF * myCell.SL.fraction_slCa * ICaL_PK_ * d * f * (1 - fCaB) * Pd.F * VFRT * (0.75 * Ki * Exp_VFRT - 0.75 * Ko) / (Exp_VFRT - 1.0);

            Itc = ICaL_Ca + ICaL_Na + ICaL_K;
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            double Vm = myTVc[Pd.IdxVm];
            double VFRT = Vm / Pd.RTF;
            double Cao = myCell.ExtCell.Cao;
            double Cai = myTVc[Pd.IdxCasl];
            double Ko = myCell.ExtCell.Ko;
            double Ki = myTVc[Pd.IdxKi];
            double Nao = myCell.ExtCell.Nao;
            double Nai = myTVc[Pd.IdxNai];
            double ISO = myCell.ExtCell.ISO;

            double ICaL_PCa_ = ICaL_PCa * (1 + 0.5 * ISO * 1000);
            double ICaL_PNa_ = ICaL_PNa * (1 + 0.5 * ISO * 1000);
            double ICaL_PK_ = ICaL_PK * (1 + 0.5 * ISO * 1000);

            d = myTVc[Pd.IdxICaL_d_sl]; 
            f = myTVc[Pd.IdxICaL_f_sl];
            fCaB = myTVc[Pd.IdxICaL_fCaB_sl];

            ICaL_Ca = SF * myCell.SL.fraction_slCa * ICaL_PCa_ * d * f * (1 - fCaB) * 4 * Pd.F * VFRT * (0.341 * Cai * Math.Exp(2 * VFRT) - 0.341 * Cao) / (Math.Exp(2 * VFRT) - 1.0);
            ICaL_Na = SF * myCell.SL.fraction_slCa * ICaL_PNa_ * d * f * (1 - fCaB) * Pd.F * VFRT * (0.75 * Nai * Math.Exp(VFRT) - 0.75 * Nao) / (Math.Exp(VFRT) - 1.0);
            ICaL_K = SF * myCell.SL.fraction_slCa * ICaL_PK_ * d * f * (1 - fCaB) * Pd.F * VFRT * (0.75 * Ki * Math.Exp(VFRT) - 0.75 * Ko) / (Math.Exp(VFRT) - 1.0);

            Itc = ICaL_Ca + ICaL_Na + ICaL_K;

            myTVc[Pd.InxICaLsl_Itc] = Itc;
            myTVc[Pd.InxICaL_Casl] = ICaL_Ca;
            myTVc[Pd.InxICaL_Nasl] = ICaL_Na;
            myTVc[Pd.InxICaL_Ksl] = ICaL_K;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpICaLsl_ListView;
            ListView.LVDispValue("ICaLsl", IxSF, ref SF);
            ListView.LVDispValue("ICaLsl", IxItc, ref Itc);

            ListView.LVDispValue("ICaLsl", Ixd, ref d);
            ListView.LVDispValue("ICaLsl", Ixd_inf, ref d_inf);
            ListView.LVDispValue("ICaLsl", Ixd_tau, ref d_tau);

            ListView.LVDispValue("ICaLsl", Ixf, ref f);
            ListView.LVDispValue("ICaLsl", Ixf_inf, ref f_inf);
            ListView.LVDispValue("ICaLsl", Ixf_tau, ref f_tau);

            ListView.LVDispValue("ICaLsl", IxfCaB, ref fCaB);

            ListView.LVDispValue("ICaLsl", IxICaL_PCa, ref ICaL_PCa);
            ListView.LVDispValue("ICaLsl", IxICaL_PNa, ref ICaL_PNa);
            ListView.LVDispValue("ICaLsl", IxICaL_PK, ref ICaL_PK);

            ListView.LVDispValue("ICaLsl", IxICaL_Ca, ref ICaL_Ca);
            ListView.LVDispValue("ICaLsl", IxICaL_Na, ref ICaL_Na);
            ListView.LVDispValue("ICaLsl", IxICaL_K, ref ICaL_K);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpICaLsl_ListView;
            ListView.LVModiValue("ICaLsl", IxSF, ref SF);
            ListView.LVModiValue("ICaLsl", IxItc, ref Itc);

            ListView.LVModiValue("ICaLsl", Ixd, ref d);
            ListView.LVModiValue("ICaLsl", Ixd_inf, ref d_inf);
            ListView.LVModiValue("ICaLsl", Ixd_tau, ref d_tau);

            ListView.LVModiValue("ICaLsl", Ixf, ref f);
            ListView.LVModiValue("ICaLsl", Ixf_inf, ref f_inf);
            ListView.LVModiValue("ICaLsl", Ixf_tau, ref f_tau);

            ListView.LVModiValue("ICaLsl", IxfCaB, ref fCaB);

            ListView.LVModiValue("ICaLsl", IxICaL_PCa, ref ICaL_PCa);
            ListView.LVModiValue("ICaLsl", IxICaL_PNa, ref ICaL_PNa);
            ListView.LVModiValue("ICaLsl", IxICaL_PK, ref ICaL_PK);

            ListView.LVModiValue("ICaLsl", IxICaL_Ca, ref ICaL_Ca);
            ListView.LVModiValue("ICaLsl", IxICaL_Na, ref ICaL_Na);
            ListView.LVModiValue("ICaLsl", IxICaL_K, ref ICaL_K);
        }
    }
}
