﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cICaTjs : cElement
    {
        private Pd.TIdx Ixd;
        private Pd.TIdx Ixd_inf;
        private Pd.TIdx Ixd_tau;

        private Pd.TIdx Ixf;
        private Pd.TIdx Ixf_inf;
        private Pd.TIdx Ixf_tau;

        private Pd.TIdx IxgCaT;

        public double d;
        public double d_inf;
        public double d_tau;

        public double f;
        public double f_inf;
        public double f_tau;

        public double gCaT = 0.4122;  // nS/pF

        public cICaTjs()  //コンストラクター
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");
            SetIx(ref IxgCaT, ref i, Pd.IntPar, "gCaT");

            SetIx(ref Ixd, ref i, Pd.IntVar, "d");
            SetIx(ref Ixd_inf, ref i, Pd.IntVar, "d_inf");
            SetIx(ref Ixd_tau, ref i, Pd.IntVar, "d_tau");

            SetIx(ref Ixf, ref i, Pd.IntVar, "f");
            SetIx(ref Ixf_inf, ref i, Pd.IntVar, "f_inf");
            SetIx(ref Ixf_tau, ref i, Pd.IntVar, "f_tau");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpICaTjs_ListView, NumOfIx);

            d = myTVc[Pd.IdxICaT_d_js];
            f = myTVc[Pd.IdxICaT_f_js];

            Itc = myTVc[Pd.InxICaTjs_Itc];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            double Vm = tvY[Pd.IdxVm];

            d_inf = 1.0 / (1.0 + Math.Exp((-30 - Vm) / 6.0));
            d_tau = 1 / (1.068 * Math.Exp((Vm + 26.3) / 30) + 1.068 * Math.Exp((-Vm - 26.3) / 30));

            d = tvY[Pd.IdxICaT_d_js];
            tvDYdt[Pd.IdxICaT_d_js] = (d_inf - d) / d_tau;

            f_inf = 1.0 / (1.0 + Math.Exp((Vm + 48) / 7));
            f_tau = 1 / (0.0153 * Math.Exp((-Vm - 61.7) / 83.3) + 0.015 * Math.Exp((Vm + 61.7) / 30));

            f = tvY[Pd.IdxICaT_f_js];
            tvDYdt[Pd.IdxICaT_f_js] = (f_inf - f) / f_tau;

            Itc = SF * myCell.JS.fraction_js * gCaT * d * f * (Vm - 45);
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            double Vm = myTVc[Pd.IdxVm];

            d = myTVc[Pd.IdxICaT_d_js];
            f = myTVc[Pd.IdxICaT_f_js];

            Itc = SF * myCell.JS.fraction_js * gCaT * d * f * (Vm - 45);

            myTVc[Pd.InxICaTjs_Itc] = Itc;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpICaTjs_ListView;
            ListView.LVDispValue("ICaTjs", IxSF, ref SF);
            ListView.LVDispValue("ICaTjs", IxItc, ref Itc);
            ListView.LVDispValue("ICaTjs", IxgCaT, ref gCaT);

            ListView.LVDispValue("ICaTjs", Ixd, ref d);
            ListView.LVDispValue("ICaTjs", Ixd_inf, ref d_inf);
            ListView.LVDispValue("ICaTjs", Ixd_tau, ref d_tau);

            ListView.LVDispValue("ICaTjs", Ixf, ref f);
            ListView.LVDispValue("ICaTjs", Ixf_inf, ref f_inf);
            ListView.LVDispValue("ICaTjs", Ixf_tau, ref f_tau);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpICaTjs_ListView;
            ListView.LVModiValue("ICaTjs", IxSF, ref SF);
            ListView.LVModiValue("ICaTjs", IxItc, ref Itc);
            ListView.LVModiValue("ICaTjs", IxgCaT, ref gCaT);

            ListView.LVModiValue("ICaTjs", Ixd, ref d);
            ListView.LVModiValue("ICaTjs", Ixd_inf, ref d_inf);
            ListView.LVModiValue("ICaTjs", Ixd_tau, ref d_tau);

            ListView.LVModiValue("ICaTjs", Ixf, ref f);
            ListView.LVModiValue("ICaTjs", Ixf_inf, ref f_inf);
            ListView.LVModiValue("ICaTjs", Ixf_tau, ref f_tau);
        }
    }
}
