﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cIKrsl : cElement
    {
        private Pd.TIdx Ixxkr;
        private Pd.TIdx Ixxkr_inf;
        private Pd.TIdx Ixxkr_tau;

        private Pd.TIdx IxgKr;

        public double xkr;
        public double xkr_inf;
        public double xkr_tau;

        public double gKr = 0.73; // nS/pF

        public cIKrsl()  //コンストラクター
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");
            SetIx(ref IxgKr, ref i, Pd.IntPar, "gKr");

            SetIx(ref Ixxkr, ref i, Pd.IntVar, "xkr");
            SetIx(ref Ixxkr_inf, ref i, Pd.IntVar, "xkr_inf");
            SetIx(ref Ixxkr_tau, ref i, Pd.IntVar, "xkr_tau");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpIKrsl_ListView, NumOfIx);

            xkr = myTVc[Pd.IdxIKr_xkr_sl];
            Itc = myTVc[Pd.InxIKrsl_Itc];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {

            double Vm = tvY[Pd.IdxVm];
            double tem = (Vm + 20.1376) / 22.1996;
            double Ko = myCell.ExtCell.Ko;

            xkr_inf = 1.0 / (1.0 + Math.Exp((Vm + 15) / -6));
            xkr_tau = 2.5 * (31.18 + 217.18 * Math.Exp(-tem * tem));

            xkr = tvY[Pd.IdxIKr_xkr_sl];
            tvDYdt[Pd.IdxIKr_xkr_sl] = (xkr_inf - xkr) / xkr_tau;

            double rkr = 1 / (1 + Math.Exp((Vm + 55) / 24));

            Itc = SF * myCell.SL.fraction_sl * gKr * Math.Sqrt(Ko / 5.4) * xkr * rkr * (Vm - myCell.Rp.K);
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            double Vm = myTVc[Pd.IdxVm];
            double Ko = myCell.ExtCell.Ko;
            double rkr = 1 / (1 + Math.Exp((Vm + 55) / 24));

            xkr = myTVc[Pd.IdxIKr_xkr_sl];

            Itc = SF * myCell.SL.fraction_sl * gKr * Math.Sqrt(Ko / 5.4) * xkr * rkr * (Vm - myCell.Rp.K);

            myTVc[Pd.InxIKrsl_Itc] = Itc;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpIKrsl_ListView;
            ListView.LVDispValue("IKrsl", IxSF, ref SF);
            ListView.LVDispValue("IKrsl", IxItc, ref Itc);
            ListView.LVDispValue("IKrsl", IxgKr, ref gKr);

            ListView.LVDispValue("IKrsl", Ixxkr, ref xkr);
            ListView.LVDispValue("IKrsl", Ixxkr_inf, ref xkr_inf);
            ListView.LVDispValue("IKrsl", Ixxkr_tau, ref xkr_tau);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpIKrsl_ListView;
            ListView.LVModiValue("IKrsl", IxSF, ref SF);
            ListView.LVModiValue("IKrsl", IxItc, ref Itc);
            ListView.LVModiValue("IKrsl", IxgKr, ref gKr);

            ListView.LVModiValue("IKrsl", Ixxkr, ref xkr);
            ListView.LVModiValue("IKrsl", Ixxkr_inf, ref xkr_inf);
            ListView.LVModiValue("IKrsl", Ixxkr_tau, ref xkr_tau);
        }
    }
}
