﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cIKsjs : cElement
    {      
        private Pd.TIdx Ixxks;
        private Pd.TIdx Ixxks_inf;
        private Pd.TIdx Ixxks_tau;
        private Pd.TIdx Ixpnak;
        private Pd.TIdx IxgKs;

        public double xks;
        public double xks_inf;
        public double xks_tau;
        public double pnak = 0.01833;

        public double gKs = 0.0035; // nS/pF

        public cIKsjs()  //コンストラクター
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");
            SetIx(ref IxgKs, ref i, Pd.IntPar, "gKs");

            SetIx(ref Ixxks, ref i, Pd.IntVar, "xks");
            SetIx(ref Ixxks_inf, ref i, Pd.IntVar, "xks_inf");
            SetIx(ref Ixxks_tau, ref i, Pd.IntVar, "xks_tau");
            SetIx(ref Ixpnak, ref i, Pd.IntPar, "pnak");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpIKsjs_ListView, NumOfIx);

            xks = myTVc[Pd.IdxIKs_xks_js];
            Itc = myTVc[Pd.InxIKsjs_Itc];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            double Vm = tvY[Pd.IdxVm];
            double Ko = myCell.ExtCell.Ko;
            double Nao = myCell.ExtCell.Nao;
            double Ki = tvY[Pd.IdxKi];
            double Nai = tvY[Pd.IdxNai];
            double ISO = myCell.ExtCell.ISO;

            double eKs = Pd.RTF * Math.Log((Ko + pnak * Nao) / (Ki + pnak * Nai));
            double gKs_ = gKs * (1 + 2 * ISO * 1000);

            xks_inf = 1.0 / (1.0 + Math.Exp((-Vm - 3.8 - 40 * ISO * 1000) / 14.25));
            xks_tau = 990.1 / (1.0 + Math.Exp((-Vm - 2.436 - 40 * ISO * 1000) / 14.12));

            xks = tvY[Pd.IdxIKs_xks_js];
            tvDYdt[Pd.IdxIKs_xks_js] = (xks_inf - xks) / xks_tau;

            Itc = SF * myCell.JS.fraction_js * gKs_ * xks * xks * (Vm - eKs);    //SF * myCell.JS.fraction_js * gKs_ * Math.Pow(xks, 2) * (Vm - eKs);
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            double Vm = myTVc[Pd.IdxVm];
            double Ko = myCell.ExtCell.Ko;
            double Nao = myCell.ExtCell.Nao;
            double Ki = myTVc[Pd.IdxKi];
            double Nai = myTVc[Pd.IdxNai];
            double ISO = myCell.ExtCell.ISO; 

            double gKs_ = gKs * (1 + 2 * ISO * 1000);
            double eKs = Pd.RTF * Math.Log((Ko + pnak * Nao) / (Ki + pnak * Nai));

            xks = myTVc[Pd.IdxIKs_xks_js];

            Itc = SF * myCell.JS.fraction_js * gKs_ * xks * xks * (Vm - eKs);    //SF * myCell.JS.fraction_js * gKs_ * Math.Pow(xks, 2) * (Vm - eKs);

            myTVc[Pd.InxIKsjs_Itc] = Itc;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpIKsjs_ListView;
            ListView.LVDispValue("IKsjs", IxSF, ref SF);
            ListView.LVDispValue("IKsjs", IxItc, ref Itc);
            ListView.LVDispValue("IKsjs", IxgKs, ref gKs);

            ListView.LVDispValue("IKsjs", Ixxks, ref xks);
            ListView.LVDispValue("IKsjs", Ixxks_inf, ref xks_inf);
            ListView.LVDispValue("IKsjs", Ixxks_tau, ref xks_tau);
            ListView.LVDispValue("IKsjs", Ixpnak, ref pnak);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpIKsjs_ListView;
            ListView.LVModiValue("IKsjs", IxSF, ref SF);
            ListView.LVModiValue("IKsjs", IxItc, ref Itc);
            ListView.LVModiValue("IKsjs", IxgKs, ref gKs);

            ListView.LVModiValue("IKsjs", Ixxks, ref xks);
            ListView.LVModiValue("IKsjs", Ixxks_inf, ref xks_inf);
            ListView.LVModiValue("IKsjs", Ixxks_tau, ref xks_tau);
            ListView.LVModiValue("IKsjs", Ixpnak, ref pnak);
        }
    }
}
