﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cIKursl : cElement
    {
        private Pd.TIdx Ixxkur;
        private Pd.TIdx Ixxkur_inf;
        private Pd.TIdx Ixxkur_tau;
        private Pd.TIdx Ixykur;
        private Pd.TIdx Ixykur_inf;
        private Pd.TIdx Ixykur_tau;
        private Pd.TIdx IxgKur;

        public double xkur;
        public double xkur_inf;
        public double xkur_tau;
        public double ykur;
        public double ykur_inf;
        public double ykur_tau;

        public double gKur = 0.035; // nS/pF

        public cIKursl()  //コンストラクター
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");
            SetIx(ref IxgKur, ref i, Pd.IntPar, "gKur");

            SetIx(ref Ixxkur, ref i, Pd.IntVar, "xkur");
            SetIx(ref Ixxkur_inf, ref i, Pd.IntVar, "xkur_inf");
            SetIx(ref Ixxkur_tau, ref i, Pd.IntVar, "xkuro_tau");
            SetIx(ref Ixykur, ref i, Pd.IntVar, "ykur");
            SetIx(ref Ixykur_inf, ref i, Pd.IntVar, "ykur_inf");
            SetIx(ref Ixykur_tau, ref i, Pd.IntVar, "ykur_tau");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpIKursl_ListView, NumOfIx);

            xkur = myTVc[Pd.IdxIKur_xkur_sl];
            ykur = myTVc[Pd.IdxIKur_ykur_sl];
            Itc = myTVc[Pd.InxIKursl_Itc];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            double Vm = tvY[Pd.IdxVm];
            double ISO = myCell.ExtCell.ISO;
            double gKur_ = gKur * (1 + 2 * ISO * 1000);

            xkur_inf = 1.0 / (1.0 + Math.Exp((-Vm - 6) / 8.6));
            xkur_tau = 0.5 + 9 / (1 + Math.Exp((Vm + 5) / 12));

            xkur = tvY[Pd.IdxIKur_xkur_sl];
            tvDYdt[Pd.IdxIKur_xkur_sl] = (xkur_inf - xkur) / xkur_tau;

            ykur_inf = 1.0 / (1.0 + Math.Exp((Vm + 7.5) / 10));
            ykur_tau = 3050 + 590 / (1 + Math.Exp((Vm + 60) / 10));

            ykur = tvY[Pd.IdxIKur_ykur_sl];
            tvDYdt[Pd.IdxIKur_ykur_sl] = (ykur_inf - ykur) / ykur_tau;

            Itc = SF * myCell.SL.fraction_sl * gKur_ * xkur * ykur * (Vm - myCell.Rp.K);
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            double Vm = myTVc[Pd.IdxVm];
            double ISO = myCell.ExtCell.ISO;
            double gKur_ = gKur * (1 + 2 * ISO * 1000);

            xkur = myTVc[Pd.IdxIKur_xkur_sl];
            ykur = myTVc[Pd.IdxIKur_ykur_sl];

            Itc = SF * myCell.SL.fraction_sl * gKur_ * xkur * ykur * (Vm - myCell.Rp.K);

            myTVc[Pd.InxIKursl_Itc] = Itc;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpIKursl_ListView;
            ListView.LVDispValue("IKursl", IxSF, ref SF);
            ListView.LVDispValue("IKursl", IxItc, ref Itc);
            ListView.LVDispValue("IKursl", IxgKur, ref gKur);

            ListView.LVDispValue("IKursl", Ixxkur, ref xkur);
            ListView.LVDispValue("IKursl", Ixxkur_inf, ref xkur_inf);
            ListView.LVDispValue("IKursl", Ixxkur_tau, ref xkur_tau);
            ListView.LVDispValue("IKursl", Ixykur, ref ykur);
            ListView.LVDispValue("IKursl", Ixykur_inf, ref ykur_inf);
            ListView.LVDispValue("IKursl", Ixykur_tau, ref ykur_tau);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpIKursl_ListView;
            ListView.LVModiValue("IKursl", IxSF, ref SF);
            ListView.LVModiValue("IKursl", IxItc, ref Itc);
            ListView.LVModiValue("IKursl", IxgKur, ref gKur);

            ListView.LVModiValue("IKursl", Ixxkur, ref xkur);
            ListView.LVModiValue("IKursl", Ixxkur_inf, ref xkur_inf);
            ListView.LVModiValue("IKursl", Ixxkur_tau, ref xkur_tau);
            ListView.LVModiValue("IKursl", Ixykur, ref ykur);
            ListView.LVModiValue("IKursl", Ixykur_inf, ref ykur_inf);
            ListView.LVModiValue("IKursl", Ixykur_tau, ref ykur_tau);
        }
    }
}
