﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cINaKjs : cElement
    {
        private Pd.TIdx IxKmKo;
        private Pd.TIdx IxKmNai;
        private Pd.TIdx IxANaK;

        public double KmKo = 1.5; // mM
        public double KmNai = 11.0; // mM
        public double ANaK = 1.26; // pA/pF

        public cINaKjs()  //コンストラクター
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");
            SetIx(ref IxANaK, ref i, Pd.IntPar, "ANaK");

            SetIx(ref IxKmKo, ref i, Pd.IntVar, "KmKo");
            SetIx(ref IxKmNai, ref i, Pd.IntVar, "KmNai");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpINaKjs_ListView, NumOfIx);

            Itc = myTVc[Pd.InxINaKjs_Itc];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            double Vm = tvY[Pd.IdxVm];
            double Ko = myCell.ExtCell.Ko;
            double Nao = myCell.ExtCell.Nao;
            double Nai = tvY[Pd.IdxNai];
            double ISO = myCell.ExtCell.ISO;

            double VFRT = Vm / Pd.RTF;
            double sigma = (Math.Exp(Nao / 67.3) - 1) / 7;
            double fNaK = 1 / (1 + 0.1245 * Math.Exp(-0.1 * VFRT) + 0.0365 * sigma * Math.Exp(-VFRT));
            double KmNai_ = KmNai - 2.2 * ISO * 1000;

            Itc = SF * myCell.JS.fraction_js * ANaK * fNaK * (Ko / (Ko + KmKo)) * (1 / (1 + Math.Pow(KmNai_ / Nai, 4)));
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            myTVc[Pd.InxINaKjs_Itc] = Itc;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpINaKjs_ListView;
            ListView.LVDispValue("INaKjs", IxSF, ref SF);
            ListView.LVDispValue("INaKjs", IxItc, ref Itc);
            ListView.LVDispValue("INaKjs", IxANaK, ref ANaK);

            ListView.LVDispValue("INaKjs", IxKmKo, ref KmKo);
            ListView.LVDispValue("INaKjs", IxKmNai, ref KmNai);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpINaKjs_ListView;
            ListView.LVModiValue("INaKjs", IxSF, ref SF);
            ListView.LVModiValue("INaKjs", IxItc, ref Itc);
            ListView.LVModiValue("INaKjs", IxANaK, ref ANaK);

            ListView.LVModiValue("INaKjs", IxKmKo, ref KmKo);
            ListView.LVModiValue("INaKjs", IxKmNai, ref KmNai);
        }
    }
}
