﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cIhajs : cElement
    {
        private Pd.TIdx Ixy;
        private Pd.TIdx Ixy_inf;
        private Pd.TIdx Ixy_tau;
        private Pd.TIdx Ixgha;
        private Pd.TIdx Ixgha_Na;
        private Pd.TIdx Ixgha_K;
        private Pd.TIdx IxIha_Na;
        private Pd.TIdx IxIha_K;

        public double y;
        public double y_inf;
        public double y_tau;
        public double gha = 0.2; // nS/pF
        public double gha_Na = 0.3833;
        public double gha_K = 0.6167;
        public double Iha_Na;
        public double Iha_K;

        public cIhajs()  //コンストラクター
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");
            SetIx(ref Ixgha, ref i, Pd.IntPar, "gha");

            SetIx(ref Ixy, ref i, Pd.IntVar, "y");
            SetIx(ref Ixy_inf, ref i, Pd.IntVar, "y_inf");
            SetIx(ref Ixy_tau, ref i, Pd.IntVar, "y_tau");
            SetIx(ref Ixgha_Na, ref i, Pd.IntPar, "gha_Na");
            SetIx(ref Ixgha_K, ref i, Pd.IntPar, "gha_K");
            SetIx(ref IxIha_Na, ref i, Pd.IntVar, "Iha_Na");
            SetIx(ref IxIha_K, ref i, Pd.IntVar, "Iha_K");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpIhajs_ListView, NumOfIx);

            y = myTVc[Pd.IdxIha_y_js];
            Itc = myTVc[Pd.InxIhajs_Itc];
            Iha_Na = myTVc[Pd.InxIha_Najs];
            Iha_K = myTVc[Pd.InxIha_Kjs];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            double Vm = tvY[Pd.IdxVm];
            double ISO = myCell.ExtCell.ISO;
            double gha_ = gha * (1 + 3.7 * ISO * 1000);

            y_inf = 1.0 / (1.0 + Math.Exp((70 - 56 * ISO * 1000 + Vm) / 13.5));
            y_tau = 1.0 / (Math.Exp((-198 - Vm) / 15) + Math.Exp((Vm - 130) / 17));

            y = tvY[Pd.IdxIha_y_js];
            tvDYdt[Pd.IdxIha_y_js] = (y_inf - y) / y_tau;

            Iha_Na = SF * myCell.JS.fraction_js * gha_ * gha_Na * (Vm - myCell.Rp.Na) * y * y;
            Iha_K = SF * myCell.JS.fraction_js * gha_ * gha_K * (Vm - myCell.Rp.K) * y * y;

            Itc = Iha_Na + Iha_K;
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            double Vm = myTVc[Pd.IdxVm];
            double ISO = myCell.ExtCell.ISO;
            double gha_ = gha * (1 + 3.7 * ISO * 1000);

            y = myTVc[Pd.IdxIha_y_js];

            Iha_Na = SF * myCell.JS.fraction_js * gha_ * gha_Na * (Vm - myCell.Rp.Na) * y * y;
            Iha_K = SF * myCell.JS.fraction_js * gha_ * gha_K * (Vm - myCell.Rp.K) * y * y;

            Itc = Iha_Na + Iha_K;

            myTVc[Pd.InxIhajs_Itc] = Itc;
            myTVc[Pd.InxIha_Najs] = Iha_Na;
            myTVc[Pd.InxIha_Kjs] = Iha_K;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpIhajs_ListView;
            ListView.LVDispValue("Ihajs", IxSF, ref SF);
            ListView.LVDispValue("Ihajs", IxItc, ref Itc);
            ListView.LVDispValue("Ihajs", Ixgha, ref gha);

            ListView.LVDispValue("Ihajs", Ixy, ref y);
            ListView.LVDispValue("Ihajs", Ixy_inf, ref y_inf);
            ListView.LVDispValue("Ihajs", Ixy_tau, ref y_tau);

            ListView.LVDispValue("Ihajs", Ixgha_Na, ref gha_Na);
            ListView.LVDispValue("Ihajs", Ixgha_K, ref gha_K);
            ListView.LVDispValue("Ihajs", IxIha_Na, ref Iha_Na);
            ListView.LVDispValue("Ihajs", IxIha_K, ref Iha_K);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpIhajs_ListView;
            ListView.LVModiValue("Ihajs", IxSF, ref SF);
            ListView.LVModiValue("Ihajs", IxItc, ref Itc);
            ListView.LVModiValue("Ihajs", Ixgha, ref gha);

            ListView.LVModiValue("Ihajs", Ixy, ref y);
            ListView.LVModiValue("Ihajs", Ixy_inf, ref y_inf);
            ListView.LVModiValue("Ihajs", Ixy_tau, ref y_tau);

            ListView.LVModiValue("Ihajs", Ixgha_Na, ref gha_Na);
            ListView.LVModiValue("Ihajs", Ixgha_K, ref gha_K);
            ListView.LVModiValue("Ihajs", IxIha_Na, ref Iha_Na);
            ListView.LVModiValue("Ihajs", IxIha_K, ref Iha_K);
        }
    }
}
