﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cItosl : cElement
    {
        private Pd.TIdx Ixxto;
        private Pd.TIdx Ixxto_inf;
        private Pd.TIdx Ixxto_tau;
        private Pd.TIdx Ixyto;
        private Pd.TIdx Ixyto_inf;
        private Pd.TIdx Ixyto_tau;
        private Pd.TIdx Ixgto;

        public double xto;
        public double xto_inf;
        public double xto_tau;
        public double yto;
        public double yto_inf;
        public double yto_tau;

        public double gto = 0.12375; // nS/pF

        public cItosl()  //コンストラクター
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");
            SetIx(ref Ixgto, ref i, Pd.IntPar, "gto");

            SetIx(ref Ixxto, ref i, Pd.IntVar, "xto");
            SetIx(ref Ixxto_inf, ref i, Pd.IntVar, "xto_inf");
            SetIx(ref Ixxto_tau, ref i, Pd.IntVar, "xto_tau");
            SetIx(ref Ixyto, ref i, Pd.IntVar, "yto");
            SetIx(ref Ixyto_inf, ref i, Pd.IntVar, "yto_inf");
            SetIx(ref Ixyto_tau, ref i, Pd.IntVar, "yto_tau");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpItosl_ListView, NumOfIx);

            xto = myTVc[Pd.IdxIto_xto_sl];
            yto = myTVc[Pd.IdxIto_yto_sl];
            Itc = myTVc[Pd.InxItosl_Itc];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            double Vm = tvY[Pd.IdxVm];

            xto_inf = 1.0 / (1.0 + Math.Exp((1 - Vm) / 11));
            xto_tau = 1.5 + 3.5 * Math.Exp(-(Vm / 30) * (Vm / 30));

            xto = tvY[Pd.IdxIto_xto_sl];
            tvDYdt[Pd.IdxIto_xto_sl] = (xto_inf - xto) / xto_tau;

            yto_inf = 1.0 / (1.0 + Math.Exp((Vm + 40.5) / 11.5));
            yto_tau = 21.21 + 38.4525 * Math.Exp(-((Vm + 52.45) / 15.8827) * ((Vm + 52.45) / 15.8827));

            yto = tvY[Pd.IdxIto_yto_sl];
            tvDYdt[Pd.IdxIto_yto_sl] = (yto_inf - yto) / yto_tau;

            Itc = SF * myCell.SL.fraction_sl * gto * xto * yto * (Vm - myCell.Rp.K);
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            double Vm = myTVc[Pd.IdxVm];

            xto = myTVc[Pd.IdxIto_xto_sl];
            yto = myTVc[Pd.IdxIto_yto_sl];

            Itc = SF * myCell.SL.fraction_sl * gto * xto * yto * (Vm - myCell.Rp.K);

            myTVc[Pd.InxItosl_Itc] = Itc;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpItosl_ListView;
            ListView.LVDispValue("Itosl", IxSF, ref SF);
            ListView.LVDispValue("Itosl", IxItc, ref Itc);
            ListView.LVDispValue("Itosl", Ixgto, ref gto);

            ListView.LVDispValue("Itosl", Ixxto, ref xto);
            ListView.LVDispValue("Itosl", Ixxto_inf, ref xto_inf);
            ListView.LVDispValue("Itosl", Ixxto_tau, ref xto_tau);
            ListView.LVDispValue("Itosl", Ixyto, ref yto);
            ListView.LVDispValue("Itosl", Ixyto_inf, ref yto_inf);
            ListView.LVDispValue("Itosl", Ixyto_tau, ref yto_tau);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpItosl_ListView;
            ListView.LVModiValue("Itosl", IxSF, ref SF);
            ListView.LVModiValue("Itosl", IxItc, ref Itc);
            ListView.LVModiValue("Itosl", Ixgto, ref gto);

            ListView.LVModiValue("Itosl", Ixxto, ref xto);
            ListView.LVModiValue("Itosl", Ixxto_inf, ref xto_inf);
            ListView.LVModiValue("Itosl", Ixxto_tau, ref xto_tau);
            ListView.LVModiValue("Itosl", Ixyto, ref yto);
            ListView.LVModiValue("Itosl", Ixyto_inf, ref yto_inf);
            ListView.LVModiValue("Itosl", Ixyto_tau, ref yto_tau);
        }
    }
}
