﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cJS: cElement
    {
        public cCabuffjs Cabuffjs = new cCabuffjs(); 
        
        private Pd.TIdx IxVjs;
        private Pd.TIdx IxVjs_part;
        private Pd.TIdx IxCajs;
        private Pd.TIdx Ixfraction_js;
        private Pd.TIdx Ixfraction_jsCa;
        private Pd.TIdx IxCajs_Min;
        private Pd.TIdx IxCajs_Max;

        public double Vjs ;
        public double Vjs_part = 0.0004; // part of cell volume occupied with junctional space
        public double Cajs;  // Ca concentration in junctional space (mM)

        public double fraction_js = 0.11;
        public double fraction_jsCa = 0.9;

        public double Cajs_Min;
        public double Cajs_Max;

        public cJS()  //コンストラクター 
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            Cajs = myTVc[Pd.IdxCajs];
            Vjs = Vjs_part * myCell.Vcell;

            Cabuffjs.Initialize(ref myTVc, ref myCell, ref Lf);

            int i = 0;

            SetIx(ref IxVjs, ref i, Pd.IntVar, "Vjs");
            SetIx(ref IxVjs_part, ref i, Pd.IntVar, "Vjs_part");
            SetIx(ref IxCajs, ref i, Pd.IntVar, "Cajs");
            SetIx(ref Ixfraction_js, ref i, Pd.IntPar, "fraction_js");
            SetIx(ref Ixfraction_jsCa, ref i, Pd.IntPar, "fraction_jsCa");
            SetIx(ref IxCajs_Min, ref i, Pd.IntVar, "Cajs_Min");
            SetIx(ref IxCajs_Max, ref i, Pd.IntVar, "Cajs_Max");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpJS_ListView, NumOfIx);
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {         
            Cabuffjs.dydt(dt, ref tvDYdt, ref tvY, myCell);

            tvDYdt[Pd.IdxCajs] = -(myCell.ICaLjs.ICaL_Ca + myCell.ICaTjs.Itc + myCell.IpCajs.Itc + myCell.ICabjs.Itc - 2 * myCell.INaCajs.Itc - myCell.SR.RyR.Itc - myCell.SR.Leak.Itc) * (myCell.Cm / (Vjs * 2 * Pd.F)) + myCell.Cadiff_jssl.JCadif_jssl / Vjs - (tvDYdt[Pd.IdxbCabuffjs_low] + tvDYdt[Pd.IdxbCabuffjs_high]);
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            Cajs = myTVc[Pd.IdxCajs];
            Cabuffjs.TransforToTV(ref myTVc, myCell);
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpJS_ListView;
            ListView.LVDispValue("JS", IxVjs, ref Vjs);
            ListView.LVDispValue("JS", IxVjs_part, ref Vjs_part);
            ListView.LVDispValue("JS", IxCajs, ref Cajs);

            ListView.LVDispValue("JS", Ixfraction_js, ref fraction_js);
            ListView.LVDispValue("JS", Ixfraction_jsCa, ref fraction_jsCa);

            ListView.LVDispValue("JS", IxCajs_Min, ref Cajs_Min);
            ListView.LVDispValue("JS", IxCajs_Max, ref Cajs_Max);

            Cabuffjs.DispValues(ref Lf);
        }
        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpJS_ListView;
            ListView.LVModiValue("JS", IxVjs, ref Vjs);
            ListView.LVModiValue("JS", IxVjs_part, ref Vjs_part);
            ListView.LVModiValue("JS", IxCajs, ref Cajs);

            ListView.LVModiValue("JS", Ixfraction_js, ref fraction_js);
            ListView.LVModiValue("JS", Ixfraction_jsCa, ref fraction_jsCa);

            ListView.LVModiValue("JS", IxCajs_Min, ref Cajs_Min);
            ListView.LVModiValue("JS", IxCajs_Max, ref Cajs_Max);

            Cabuffjs.ModiValues(ref Lf);
        }
    }
}
