﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cMitochondria : cElement
    {
        public cCaUni CaUni = new cCaUni();
        public cNCXmit NCXmit = new cNCXmit();

        private Pd.TIdx IxVmit;
        private Pd.TIdx IxVmit_part;
        private Pd.TIdx IxNamit;
        private Pd.TIdx IxdPsi;
        private Pd.TIdx IxKm;
        private Pd.TIdx Ixbuff_tot;
        private Pd.TIdx IxCamit_tot;
        private Pd.TIdx IxCamit;
        private Pd.TIdx IxCamit_Min;
        private Pd.TIdx IxCamit_Max;

        public double Vmit;
        public double Vmit_part = 0.2; // part of cell volume occupied with mitochondria
        public double Namit;  // Na concentration in mitochondria (mM)
        public double dPsi;  // mitochondrial membrane potential (mV)
        public double Km = 0.001;  // mM
        public double buff_tot = 0.6; // /mM
        public double Camit_tot; // total Ca concentration in mitochondria (mM)
        public double Camit;  // free Ca concentration in mitochondria (mM)
        public double Camit_Min;
        public double Camit_Max;

        public cMitochondria()  //コンストラクター 
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            Camit_tot = myTVc[Pd.IdxCamit_tot];
            Camit = myTVc[Pd.InxCamit];

            Namit = 4.0;
            dPsi = -150.0;
            Vmit = Vmit_part * myCell.Vcell;

            CaUni.Initialize(ref myTVc, ref myCell, ref Lf);
            NCXmit.Initialize(ref myTVc, ref myCell, ref Lf);

            int i = 0;

            SetIx(ref IxVmit, ref i, Pd.IntVar, "Vmit");
            SetIx(ref IxVmit_part, ref i, Pd.IntVar, "Vmit_part");
            SetIx(ref IxNamit, ref i, Pd.IntVar, "Namit");
            SetIx(ref IxdPsi, ref i, Pd.IntPar, "dPsi");
            SetIx(ref IxKm, ref i, Pd.IntVar, "Km");
            SetIx(ref Ixbuff_tot, ref i, Pd.IntPar, "buff_tot");
            SetIx(ref IxCamit_tot, ref i, Pd.IntVar, "Camit_tot");
            SetIx(ref IxCamit, ref i, Pd.IntVar, "Camit");
            SetIx(ref IxCamit_Min, ref i, Pd.IntPar, "Camit_Min");
            SetIx(ref IxCamit_Max, ref i, Pd.IntPar, "Camit_Max");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpMitochondria_ListView, NumOfIx);
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            CaUni.dydt(dt, ref tvDYdt, ref tvY, myCell);
            NCXmit.dydt(dt, ref tvDYdt, ref tvY, myCell);

            tvDYdt[Pd.IdxCamit_tot] = -(myCell.Mitochondria.CaUni.Itc - 2 * myCell.Mitochondria.NCXmit.Itc) * (myCell.Cm / (Vmit * 2 * Pd.F));
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            CaUni.TransforToTV(ref myTVc, myCell);
            NCXmit.TransforToTV(ref myTVc, myCell);

            Camit_tot = myCell.TVc[Pd.IdxCamit_tot];

            double b1 = buff_tot - Camit_tot + Km;
            double c1 = Km * Camit_tot;
            double freeCa = (Math.Sqrt(b1 * b1 + 4 * c1) - b1) * 0.5;

            myCell.TVc[Pd.InxCamit] = freeCa;
            Camit = freeCa;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpMitochondria_ListView;
            ListView.LVDispValue("Mit", IxVmit, ref Vmit);
            ListView.LVDispValue("Mit", IxVmit_part, ref Vmit_part);
            ListView.LVDispValue("Mit", IxNamit, ref Namit);
            ListView.LVDispValue("Mit", IxdPsi, ref dPsi);
            ListView.LVDispValue("Mit", IxKm, ref Km);
            ListView.LVDispValue("Mit", Ixbuff_tot, ref buff_tot);
            ListView.LVDispValue("Mit", IxCamit_tot, ref Camit_tot);
            ListView.LVDispValue("Mit", IxCamit, ref Camit);

            ListView.LVDispValue("Mit", IxCamit_Min, ref Camit_Min);
            ListView.LVDispValue("Mit", IxCamit_Max, ref Camit_Max);

            CaUni.DispValues(ref Lf);
            NCXmit.DispValues(ref Lf);
        }
        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpMitochondria_ListView;
            ListView.LVModiValue("Mit", IxVmit, ref Vmit);
            ListView.LVModiValue("Mit", IxVmit_part, ref Vmit_part);
            ListView.LVModiValue("Mit", IxNamit, ref Namit);
            ListView.LVModiValue("Mit", IxdPsi, ref dPsi);
            ListView.LVModiValue("Mit", IxKm, ref Km);
            ListView.LVModiValue("Mit", Ixbuff_tot, ref buff_tot);
            ListView.LVModiValue("Mit", IxCamit_tot, ref Camit_tot);
            ListView.LVModiValue("Mit", IxCamit, ref Camit);

            ListView.LVModiValue("Mit", IxCamit_Min, ref Camit_Min);
            ListView.LVModiValue("Mit", IxCamit_Max, ref Camit_Max);

            CaUni.ModiValues(ref Lf);
            NCXmit.ModiValues(ref Lf);
        }
    }
}
