﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cNCXmit : cElement
    {
        private Pd.TIdx IxKdNamit;
        private Pd.TIdx IxKdNai;
        private Pd.TIdx IxKdCamit;
        private Pd.TIdx IxKdCai;
        private Pd.TIdx Ixn;
        private Pd.TIdx Ixpartition;
        private Pd.TIdx IxANCXmit;
        private Pd.TIdx IxJNCXmit;

        public double KdNamit = 38.0; // mM
        public double KdNai = 32.0; // mM
        public double KdCamit = 0.020932007; // mM
        public double KdCai = 0.0125; // mM
        public double n = 3.0;
        public double partition = 0.2;
        public double ANCXmit = 0.105096;
        public double JNCXmit;

        public cNCXmit()  //コンストラクター 
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");
            SetIx(ref IxKdNamit, ref i, Pd.IntPar, "KdNamit");
            SetIx(ref IxKdNai, ref i, Pd.IntPar, "KdNai");
            SetIx(ref IxKdCamit, ref i, Pd.IntPar, "KdCamit");
            SetIx(ref IxKdCai, ref i, Pd.IntPar, "KdCai");
            SetIx(ref Ixn, ref i, Pd.IntPar, "n");
            SetIx(ref Ixpartition, ref i, Pd.IntPar, "partition");
            SetIx(ref IxANCXmit, ref i, Pd.IntPar, "ANCXmit");
            SetIx(ref IxJNCXmit, ref i, Pd.IntVar, "JNCXmit");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpNCXmit_ListView, NumOfIx);

            Itc = myTVc[Pd.InxNCXmit_Itc];
            JNCXmit = myTVc[Pd.InxJNCXmit];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            double dPsi = myCell.Mitochondria.dPsi;
            double Cai = tvY[Pd.IdxCai];
            double Camit = myCell.Mitochondria.Camit;
            double Nai = tvY[Pd.IdxNai];
            double Namit = myCell.Mitochondria.Namit;
            double Vmit = myCell.Mitochondria.Vmit;
            double FRT = 1/Pd.RTF;
            double Namit_n = Namit * Namit * Namit; // Math.Pow(Namit, n);
            double Nai_n = Nai * Nai * Nai;  // Math.Pow(Nai, n);


            double NamitOverKdNamit = Namit / KdNamit;
            double NaiOverKdNai = Nai / KdNai;

            double PNamit = Namit_n / (Namit_n + KdNamit * KdNamit * KdNamit * (1 + Camit / KdCamit)); //Namit_n /( Namit_n + Math.Pow(KdNamit, n) * (1 + Camit / KdCamit));
            double PCamit = Camit / (Camit + KdCamit * (1 + NamitOverKdNamit * NamitOverKdNamit * NamitOverKdNamit));   // Camit / (Camit + KdCamit * (1 + Math.Pow((Namit / KdNamit), n)));
            double PNai = Nai_n / (Nai_n + KdNai * KdNai * KdNai * (1 + Cai / KdCai)); //Nai_n / (Nai_n + Math.Pow(KdNai, n) * (1 + Cai / KdCai));
            double PCai = Cai / (Cai + KdCai * (1 + NaiOverKdNai * NaiOverKdNai * NaiOverKdNai));   //PCai = Cai / (Cai + KdCai * (1 + Math.Pow((Nai / KdNai), n)));

            double k1 = 1.0 * Math.Exp(partition * dPsi * FRT) * PNamit;			//    /msec
            double k2 = 1.0 * Math.Exp((partition - 1.0) * dPsi * FRT) * PNai;		//    /msec
            double k3 = 1.0 * PCamit;												//    /msec
            double k4 = 1.0 * PCai;													//    /msec

            double alpha = k2 + k4;
            double beta = k1 + k3;

            double tE1_ = alpha / (alpha + beta);

            if (tE1_ < 0)
            {
                tE1_ = 0;
            }
            else
            {
                tE1_ = alpha / (alpha + beta);
            }
            double tE2_ = 1.0 - tE1_;

            Itc = SF * (Pd.F * 2 / myCell.Cm) * (-ANCXmit) * (tE2_ * k2 - tE1_ * k1);
            JNCXmit = (myCell.Cm / (Vmit * Pd.F * 2)) * Itc;
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            myTVc[Pd.InxNCXmit_Itc] = Itc;
            myTVc[Pd.InxJNCXmit] = JNCXmit;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpNCXmit_ListView;
            ListView.LVDispValue("NCXmit", IxSF, ref SF);
            ListView.LVDispValue("NCXmit", IxItc, ref Itc);
            ListView.LVDispValue("NCXmit", IxKdNamit, ref KdNamit);
            ListView.LVDispValue("NCXmit", IxKdNai, ref KdNai);
            ListView.LVDispValue("NCXmit", IxKdCamit, ref KdCamit);
            ListView.LVDispValue("NCXmit", IxKdCai, ref KdCai);
            ListView.LVDispValue("NCXmit", Ixn, ref n);
            ListView.LVDispValue("NCXmit", Ixpartition, ref partition);
            ListView.LVDispValue("NCXmit", IxANCXmit, ref ANCXmit);
            ListView.LVDispValue("NCXmit", IxJNCXmit, ref JNCXmit);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpNCXmit_ListView;
            ListView.LVModiValue("NCXmit", IxSF, ref SF);
            ListView.LVModiValue("NCXmit", IxItc, ref Itc);
            ListView.LVModiValue("NCXmit", IxKdNamit, ref KdNamit);
            ListView.LVModiValue("NCXmit", IxKdNai, ref KdNai);
            ListView.LVModiValue("NCXmit", IxKdCamit, ref KdCamit);
            ListView.LVModiValue("NCXmit", IxKdCai, ref KdCai);
            ListView.LVModiValue("NCXmit", Ixn, ref n);
            ListView.LVModiValue("NCXmit", Ixpartition, ref partition);
            ListView.LVModiValue("NCXmit", IxANCXmit, ref ANCXmit);
            ListView.LVModiValue("NCXmit", IxJNCXmit, ref JNCXmit);
        }
    }
}
