﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HL_1
{
    public class cSERCA : cElement
    {
        private Pd.TIdx IxVmax_SERCA;
        private Pd.TIdx IxKmf;
        private Pd.TIdx IxKmr;
        private Pd.TIdx IxJSERCA;

        public double Vmax_SERCA = 0.015; // mM/ms
        public double Kmf = 4.92e-4;
        public double Kmr = 1.7;
        public double JSERCA;

        public cSERCA()  //コンストラクター
        {
        }

        override public void Initialize(ref double[] myTVc, ref cCell myCell, ref ListForm Lf)
        {
            int i = 0;

            SetIx(ref IxSF, ref i, Pd.IntPar, "SF");
            SetIx(ref IxItc, ref i, Pd.IntVar, "Itc");
            SetIx(ref IxVmax_SERCA, ref i, Pd.IntPar, "Vmax_SERCA");

            SetIx(ref IxKmf, ref i, Pd.IntPar, "Kmf_SERCA");
            SetIx(ref IxKmr, ref i, Pd.IntPar, "Kmr_SERCA");
            SetIx(ref IxJSERCA, ref i, Pd.IntVar, "JSERCA");

            NumOfIx = i;

            Lf.DGViews_Initialize(Lf.tpSERCA_ListView, NumOfIx);

            Itc = myTVc[Pd.InxSERCA_Itc];
            JSERCA = myTVc[Pd.InxJSERCA];
        }

        override public void dydt(double dt, ref double[] tvDYdt, ref double[] tvY, cCell myCell)
        {
            double Cai = tvY[Pd.IdxCai];
            double Casr = tvY[Pd.IdxCasr];
            double ISO = myCell.ExtCell.ISO;
            double Vsr = myCell.SR.Vsr;

            double Kmf_ = Kmf * (1 - 0.5 * ISO * 1000);

            JSERCA = SF * Vmax_SERCA * (Math.Pow(Cai / Kmf_, 1.787) - Math.Pow(Casr / Kmr, 1.787)) / (1 + Math.Pow(Cai / Kmf_, 1.787) + Math.Pow(Casr / Kmr, 1.787));
            Itc = (Vsr * Pd.F * 2 / myCell.Cm) * JSERCA;
        }

        override public void TransforToTV(ref double[] myTVc, cCell myCell)
        {
            myTVc[Pd.InxSERCA_Itc] = Itc;
            myTVc[Pd.InxJSERCA] = JSERCA;
        }

        override public void DispValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpSERCA_ListView;
            ListView.LVDispValue("SERCA", IxSF, ref SF);
            ListView.LVDispValue("SERCA", IxItc, ref Itc);
            ListView.LVDispValue("SERCA", IxVmax_SERCA, ref Vmax_SERCA);

            ListView.LVDispValue("SERCA", IxKmf, ref Kmf);
            ListView.LVDispValue("SERCA", IxKmr, ref Kmr);
            ListView.LVDispValue("SERCA", IxJSERCA, ref JSERCA);
        }

        override public void ModiValues(ref ListForm Lf)
        {
            ucListView ListView = Lf.tpSERCA_ListView;
            ListView.LVModiValue("SERCA", IxSF, ref SF);
            ListView.LVModiValue("SERCA", IxItc, ref Itc);
            ListView.LVModiValue("SERCA", IxVmax_SERCA, ref Vmax_SERCA);

            ListView.LVModiValue("SERCA", IxKmf, ref Kmf);
            ListView.LVModiValue("SERCA", IxKmr, ref Kmr);
            ListView.LVModiValue("SERCA", IxJSERCA, ref JSERCA);
        }
    }
}
